@@ -22,9 +22,9 @@
 
 #include "config.h"
 
+#include "libglnx.h"
 #include "ostree.h"
 #include "otutil.h"
-#include "libgsystem.h"
 
 static gboolean
 get_file_checksum (OstreeDiffFlags  flags,
@@ -35,8 +35,8 @@ get_file_checksum (OstreeDiffFlags  flags,
                    GError   **error)
 {
   gboolean ret = FALSE;
-  gs_free char *ret_checksum = NULL;
-  gs_free guchar *csum = NULL;
+  g_autofree char *ret_checksum = NULL;
+  g_autofree guchar *csum = NULL;
 
   if (OSTREE_IS_REPO_FILE (f))
     {
@@ -44,12 +44,13 @@ get_file_checksum (OstreeDiffFlags  flags,
     }
   else
     {
-      gs_unref_variant GVariant *xattrs = NULL;
-      gs_unref_object GInputStream *in = NULL;
+      g_autoptr(GVariant) xattrs = NULL;
+      g_autoptr(GInputStream) in = NULL;
 
       if (!(flags & OSTREE_DIFF_FLAGS_IGNORE_XATTRS))
         {
-          if (!gs_file_get_all_xattrs (f, &xattrs, cancellable, error))
+          if (!glnx_dfd_name_get_all_xattrs (AT_FDCWD, gs_file_get_path_cached (f),
+                                             &xattrs, cancellable, error))
             goto out;
         }
 
@@ -129,8 +130,8 @@ diff_files (OstreeDiffFlags  flags,
             GError         **error)
 {
   gboolean ret = FALSE;
-  gs_free char *checksum_a = NULL;
-  gs_free char *checksum_b = NULL;
+  g_autofree char *checksum_a = NULL;
+  g_autofree char *checksum_b = NULL;
   OstreeDiffItem *ret_item = NULL;
 
   if (!get_file_checksum (flags, a, a_info, &checksum_a, cancellable, error))
@@ -160,9 +161,9 @@ diff_add_dir_recurse (GFile          *d,
 {
   gboolean ret = FALSE;
   GError *temp_error = NULL;
-  gs_unref_object GFileEnumerator *dir_enum = NULL;
-  gs_unref_object GFile *child = NULL;
-  gs_unref_object GFileInfo *child_info = NULL;
+  g_autoptr(GFileEnumerator) dir_enum = NULL;
+  g_autoptr(GFile) child = NULL;
+  g_autoptr(GFileInfo) child_info = NULL;
 
   dir_enum = g_file_enumerate_children (d, OSTREE_GIO_FAST_QUERYINFO, 
                                         G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
@@ -204,11 +205,13 @@ diff_add_dir_recurse (GFile          *d,
 /**
  * ostree_diff_dirs:
  * @flags: Flags
- * @a: First directory path
+ * @a: First directory path, or %NULL
  * @b: First directory path
  * @modified: (element-type OstreeDiffItem): Modified files
  * @removed: (element-type Gio.File): Removed files
  * @added: (element-type Gio.File): Added files
+ * @cancellable: Cancellable
+ * @error: Error
  *
  * Compute the difference between directory @a and @b as 3 separate
  * sets of #OstreeDiffItem in @modified, @removed, and @added.
@@ -225,11 +228,20 @@ ostree_diff_dirs (OstreeDiffFlags flags,
 {
   gboolean ret = FALSE;
   GError *temp_error = NULL;
-  gs_unref_object GFileEnumerator *dir_enum = NULL;
-  gs_unref_object GFile *child_a = NULL;
-  gs_unref_object GFile *child_b = NULL;
-  gs_unref_object GFileInfo *child_a_info = NULL;
-  gs_unref_object GFileInfo *child_b_info = NULL;
+  g_autoptr(GFileEnumerator) dir_enum = NULL;
+  g_autoptr(GFile) child_a = NULL;
+  g_autoptr(GFile) child_b = NULL;
+  g_autoptr(GFileInfo) child_a_info = NULL;
+  g_autoptr(GFileInfo) child_b_info = NULL;
+
+  if (a == NULL)
+    {
+      if (!diff_add_dir_recurse (b, added, cancellable, error))
+        goto out;
+
+      ret = TRUE;
+      goto out;
+    }
 
   child_a_info = g_file_query_info (a, OSTREE_GIO_FAST_QUERYINFO,
                                     G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
@@ -403,7 +415,7 @@ print_diff_item (char        prefix,
 {
   if (g_file_is_native (file))
     {
-      gs_free char *relpath = g_file_get_relative_path (base, file);
+      g_autofree char *relpath = g_file_get_relative_path (base, file);
       g_print ("%c    %s\n", prefix, relpath);
     }
   else
