@@ -24,6 +24,10 @@
 
 #include <gio/gfiledescriptorbased.h>
 #include <gio/gunixoutputstream.h>
+#define LIBSOUP_USE_UNSTABLE_REQUEST_API
+#include <libsoup/soup.h>
+#include <libsoup/soup-requester.h>
+#include <libsoup/soup-request-http.h>
 
 #include "libglnx.h"
 #include "ostree-fetcher.h"
@@ -53,6 +57,7 @@ typedef struct {
   GLnxLockFile tmpdir_lock;
   int base_tmpdir_dfd;
 
+  GVariant *extra_headers;
   int max_outstanding;
 
   /* Queue for libsoup, see bgo#708591 */
@@ -65,6 +70,9 @@ typedef struct {
 
   /* Also protected by output_stream_set_lock. */
   guint64 total_downloaded;
+
+  GError *oob_error;
+
 } ThreadClosure;
 
 static void
@@ -124,11 +132,10 @@ G_DEFINE_TYPE (OstreeFetcher, _ostree_fetcher, G_TYPE_OBJECT)
 static ThreadClosure *
 thread_closure_ref (ThreadClosure *thread_closure)
 {
+  int refcount;
   g_return_val_if_fail (thread_closure != NULL, NULL);
-  g_return_val_if_fail (thread_closure->ref_count > 0, NULL);
-
-  g_atomic_int_inc (&thread_closure->ref_count);
-
+  refcount = g_atomic_int_add (&thread_closure->ref_count, 1);
+  g_assert (refcount > 0);
   return thread_closure;
 }
 
@@ -136,7 +143,6 @@ static void
 thread_closure_unref (ThreadClosure *thread_closure)
 {
   g_return_if_fail (thread_closure != NULL);
-  g_return_if_fail (thread_closure->ref_count > 0);
 
   if (g_atomic_int_dec_and_test (&thread_closure->ref_count))
     {
@@ -145,6 +151,8 @@ thread_closure_unref (ThreadClosure *thread_closure)
 
       g_clear_pointer (&thread_closure->main_context, g_main_context_unref);
 
+      g_clear_pointer (&thread_closure->extra_headers, (GDestroyNotify)g_variant_unref);
+
       if (thread_closure->tmpdir_dfd != -1)
         close (thread_closure->tmpdir_dfd);
 
@@ -159,6 +167,8 @@ thread_closure_unref (ThreadClosure *thread_closure)
       g_clear_pointer (&thread_closure->output_stream_set, g_hash_table_unref);
       g_mutex_clear (&thread_closure->output_stream_set_lock);
 
+      g_clear_pointer (&thread_closure->oob_error, g_error_free);
+
       g_slice_free (ThreadClosure, thread_closure);
     }
 }
@@ -189,11 +199,10 @@ pending_task_compare (gconstpointer a,
 static OstreeFetcherPendingURI *
 pending_uri_ref (OstreeFetcherPendingURI *pending)
 {
+  gint refcount;
   g_return_val_if_fail (pending != NULL, NULL);
-  g_return_val_if_fail (pending->ref_count > 0, NULL);
-
-  g_atomic_int_inc (&pending->ref_count);
-
+  refcount = g_atomic_int_add (&pending->ref_count, 1);
+  g_assert (refcount > 0);
   return pending;
 }
 
@@ -276,6 +285,29 @@ session_thread_config_flags (ThreadClosure *thread_closure,
     }
 }
 
+static void
+on_authenticate (SoupSession *session, SoupMessage *msg, SoupAuth *auth,
+                 gboolean retrying, gpointer user_data)
+{
+  ThreadClosure *thread_closure = user_data;
+
+  if (msg->status_code == SOUP_STATUS_PROXY_UNAUTHORIZED)
+    {
+      SoupURI *uri = NULL;
+      g_object_get (session, SOUP_SESSION_PROXY_URI, &uri, NULL);
+      if (retrying)
+        {
+          g_autofree char *s = soup_uri_to_string (uri, FALSE);
+          g_set_error (&thread_closure->oob_error,
+                       G_IO_ERROR, G_IO_ERROR_PROXY_AUTH_FAILED,
+                       "Invalid username or password for proxy '%s'", s);
+        }
+      else
+        soup_auth_authenticate (auth, soup_uri_get_user (uri),
+                                      soup_uri_get_password (uri));
+    }
+}
+
 static void
 session_thread_set_proxy_cb (ThreadClosure *thread_closure,
                              gpointer data)
@@ -285,6 +317,37 @@ session_thread_set_proxy_cb (ThreadClosure *thread_closure,
   g_object_set (thread_closure->session,
                 SOUP_SESSION_PROXY_URI,
                 proxy_uri, NULL);
+
+  /* libsoup won't necessarily pass any embedded username and password to proxy
+   * requests, so we have to be ready to handle 407 and handle them ourselves.
+   * See also: https://bugzilla.gnome.org/show_bug.cgi?id=772932
+   * */
+  if (soup_uri_get_user (proxy_uri) &&
+      soup_uri_get_password (proxy_uri))
+    {
+      g_signal_connect (thread_closure->session, "authenticate",
+                        G_CALLBACK (on_authenticate), thread_closure);
+    }
+}
+
+static void
+session_thread_set_cookie_jar_cb (ThreadClosure *thread_closure,
+                                  gpointer data)
+{
+  SoupCookieJar *jar = data;
+
+  soup_session_add_feature (thread_closure->session,
+                            SOUP_SESSION_FEATURE (jar));
+}
+
+static void
+session_thread_set_headers_cb (ThreadClosure *thread_closure,
+                               gpointer data)
+{
+  GVariant *headers = data;
+
+  g_clear_pointer (&thread_closure->extra_headers, (GDestroyNotify)g_variant_unref);
+  thread_closure->extra_headers = g_variant_ref (headers);
 }
 
 #ifdef HAVE_LIBSOUP_CLIENT_CERTS
@@ -359,25 +422,20 @@ static void
 create_pending_soup_request (OstreeFetcherPendingURI  *pending,
                              GError                  **error)
 {
-  g_autofree char *uristr = NULL;
-  SoupURI *next_mirror = NULL;
-  SoupURI *uri = NULL;
+  OstreeFetcherURI *next_mirror = NULL;
+  g_autoptr(OstreeFetcherURI) uri = NULL;
 
   g_assert (pending->mirrorlist);
   g_assert (pending->mirrorlist_idx < pending->mirrorlist->len);
 
-  next_mirror = g_ptr_array_index (pending->mirrorlist,
-                                   pending->mirrorlist_idx);
-  uristr = g_build_filename (soup_uri_get_path (next_mirror),
-                             pending->filename /* may be NULL */, NULL);
-  uri = soup_uri_copy (next_mirror);
-  soup_uri_set_path (uri, uristr);
+  next_mirror = g_ptr_array_index (pending->mirrorlist, pending->mirrorlist_idx);
+  if (pending->filename)
+    uri = _ostree_fetcher_uri_new_subpath (next_mirror, pending->filename);
 
   g_clear_object (&pending->request);
 
   pending->request = soup_session_request_uri (pending->thread_closure->session,
-                                               uri, error);
-  soup_uri_free (uri);
+                                               (SoupURI*)(uri ? uri : next_mirror), error);
 }
 
 static void
@@ -399,6 +457,17 @@ session_thread_request_uri (ThreadClosure *thread_closure,
       return;
     }
 
+  if (SOUP_IS_REQUEST_HTTP (pending->request) && thread_closure->extra_headers)
+    {
+      glnx_unref_object SoupMessage *msg = soup_request_http_get_message ((SoupRequestHTTP*) pending->request);
+      g_autoptr(GVariantIter) viter = g_variant_iter_new (thread_closure->extra_headers);
+      const char *key;
+      const char *value;
+
+      while (g_variant_iter_next (viter, "(&s&s)", &key, &value))
+        soup_message_headers_append (msg->request_headers, key, value);
+    }
+
   if (pending->is_stream)
     {
       soup_request_send_async (pending->request,
@@ -707,6 +776,23 @@ _ostree_fetcher_set_proxy (OstreeFetcher *self,
     }
 }
 
+void
+_ostree_fetcher_set_cookie_jar (OstreeFetcher *self,
+                                const char    *jar_path)
+{
+  SoupCookieJar *jar;
+
+  g_return_if_fail (OSTREE_IS_FETCHER (self));
+  g_return_if_fail (jar_path != NULL);
+
+  jar = soup_cookie_jar_text_new (jar_path, TRUE);
+
+  session_thread_idle_add (self->thread_closure,
+                           session_thread_set_cookie_jar_cb,
+                           jar,  /* takes ownership */
+                           (GDestroyNotify) g_object_unref);
+}
+
 void
 _ostree_fetcher_set_client_cert (OstreeFetcher   *self,
                                  GTlsCertificate *cert)
@@ -746,6 +832,16 @@ _ostree_fetcher_set_tls_database (OstreeFetcher *self,
     }
 }
 
+void
+_ostree_fetcher_set_extra_headers (OstreeFetcher *self,
+                                   GVariant      *extra_headers)
+{
+  session_thread_idle_add (self->thread_closure,
+                           session_thread_set_headers_cb,
+                           g_variant_ref (extra_headers),
+                           (GDestroyNotify) g_variant_unref);
+}
+
 static gboolean
 finish_stream (OstreeFetcherPendingURI *pending,
                GCancellable            *cancellable,
@@ -991,6 +1087,7 @@ on_request_sent (GObject        *object,
               switch (msg->status_code)
                 {
                 case 404:
+                case 403:
                 case 410:
                   code = G_IO_ERROR_NOT_FOUND;
                   break;
@@ -998,10 +1095,23 @@ on_request_sent (GObject        *object,
                   code = G_IO_ERROR_FAILED;
                 }
 
-              local_error = g_error_new (G_IO_ERROR, code,
-                                         "Server returned status %u: %s",
-                                         msg->status_code,
-                                         soup_status_get_phrase (msg->status_code));
+              {
+                g_autofree char *errmsg =
+                  g_strdup_printf ("Server returned status %u: %s",
+                                   msg->status_code,
+                                   soup_status_get_phrase (msg->status_code));
+
+                /* Let's make OOB errors be the final one since they're probably
+                 * the cause for the error here. */
+                if (pending->thread_closure->oob_error)
+                  {
+                    local_error =
+                      g_error_copy (pending->thread_closure->oob_error);
+                    g_prefix_error (&local_error, "%s: ", errmsg);
+                  }
+                else
+                  local_error = g_error_new_literal (G_IO_ERROR, code, errmsg);
+              }
 
               if (pending->mirrorlist->len > 1)
                 g_prefix_error (&local_error,
@@ -1230,7 +1340,6 @@ _ostree_fetcher_mirrored_request_to_membuf (OstreeFetcher  *fetcher,
 {
   gboolean ret = FALSE;
   const guint8 nulchar = 0;
-  g_autofree char *ret_contents = NULL;
   g_autoptr(GMemoryOutputStream) buf = NULL;
   g_autoptr(GMainContext) mainctx = NULL;
   FetchUriSyncData data;
@@ -1294,7 +1403,7 @@ _ostree_fetcher_mirrored_request_to_membuf (OstreeFetcher  *fetcher,
 /* Helper for callers who just want to fetch single one-off URIs */
 gboolean
 _ostree_fetcher_request_uri_to_membuf (OstreeFetcher  *fetcher,
-                                       SoupURI        *uri,
+                                       OstreeFetcherURI *uri,
                                        gboolean        add_nul,
                                        gboolean        allow_noent,
                                        GBytes         **out_contents,
@@ -1309,3 +1418,84 @@ _ostree_fetcher_request_uri_to_membuf (OstreeFetcher  *fetcher,
                                                      out_contents, max_size,
                                                      cancellable, error);
 }
+
+void
+_ostree_fetcher_uri_free (OstreeFetcherURI *uri)
+{
+  if (uri)
+    soup_uri_free ((SoupURI*)uri);
+}
+
+OstreeFetcherURI *
+_ostree_fetcher_uri_parse (const char       *str,
+                           GError          **error)
+{
+  SoupURI *soupuri = soup_uri_new (str);
+  if (soupuri == NULL)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Failed to parse uri: %s", str);
+      return NULL;
+    }
+  return (OstreeFetcherURI*)soupuri;
+}
+
+static OstreeFetcherURI *
+_ostree_fetcher_uri_new_path_internal (OstreeFetcherURI *uri,
+                                       gboolean          extend,
+                                       const char       *path)
+{
+  SoupURI *newuri = soup_uri_copy ((SoupURI*)uri);
+  if (path)
+    {
+      if (extend)
+        {
+          const char *origpath = soup_uri_get_path ((SoupURI*)uri);
+          g_autofree char *newpath = g_build_filename (origpath, path, NULL);
+          soup_uri_set_path (newuri, newpath);
+        }
+      else
+        {
+          soup_uri_set_path (newuri, path);
+        }
+    }
+  return (OstreeFetcherURI*)newuri;
+}
+
+OstreeFetcherURI *
+_ostree_fetcher_uri_new_path (OstreeFetcherURI *uri,
+                              const char       *path)
+{
+  return _ostree_fetcher_uri_new_path_internal (uri, FALSE, path);
+}
+
+OstreeFetcherURI *
+_ostree_fetcher_uri_new_subpath (OstreeFetcherURI *uri,
+                                 const char       *subpath)
+{
+  return _ostree_fetcher_uri_new_path_internal (uri, TRUE, subpath);
+}
+
+OstreeFetcherURI *
+_ostree_fetcher_uri_clone (OstreeFetcherURI *uri)
+{
+  return _ostree_fetcher_uri_new_subpath (uri, NULL);
+}
+
+char *
+_ostree_fetcher_uri_get_scheme (OstreeFetcherURI *uri)
+{
+  return g_strdup (soup_uri_get_scheme ((SoupURI*)uri));
+}
+
+char *
+_ostree_fetcher_uri_get_path (OstreeFetcherURI *uri)
+{
+  return g_strdup (soup_uri_get_path ((SoupURI*)uri));
+}
+
+char *
+_ostree_fetcher_uri_to_string (OstreeFetcherURI *uri)
+{
+  return soup_uri_to_string ((SoupURI*)uri, FALSE);
+}
