@@ -25,10 +25,9 @@
 #include "ostree-mutable-tree.h"
 #include "otutil.h"
 #include "ostree-core.h"
-#include "libgsystem.h"
 
 /**
- * SECTION:libostree-mutable-tree
+ * SECTION:ostree-mutable-tree
  * @title: In-memory modifiable filesystem tree
  * @short_description: Modifiable filesystem tree
  *
@@ -160,6 +159,11 @@ ostree_mutable_tree_replace_file (OstreeMutableTree *self,
 {
   gboolean ret = FALSE;
 
+  g_return_val_if_fail (name != NULL, FALSE);
+
+  if (!ot_util_filename_validate (name, error))
+    goto out;
+
   if (g_hash_table_lookup (self->subdirs, name))
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
@@ -184,10 +188,13 @@ ostree_mutable_tree_ensure_dir (OstreeMutableTree *self,
                                 GError           **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object OstreeMutableTree *ret_dir = NULL;
+  glnx_unref_object OstreeMutableTree *ret_dir = NULL;
 
   g_return_val_if_fail (name != NULL, FALSE);
 
+  if (!ot_util_filename_validate (name, error))
+    goto out;
+
   if (g_hash_table_lookup (self->files, name))
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
@@ -217,8 +224,8 @@ ostree_mutable_tree_lookup (OstreeMutableTree   *self,
                             GError             **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object OstreeMutableTree *ret_subdir = NULL;
-  gs_free char *ret_file_checksum = NULL;
+  glnx_unref_object OstreeMutableTree *ret_subdir = NULL;
+  g_autofree char *ret_file_checksum = NULL;
   
   ret_subdir = ot_gobject_refz (g_hash_table_lookup (self->subdirs, name));
   if (!ret_subdir)
@@ -259,7 +266,7 @@ ostree_mutable_tree_ensure_parent_dirs (OstreeMutableTree  *self,
   gboolean ret = FALSE;
   int i;
   OstreeMutableTree *subdir = self; /* nofree */
-  gs_unref_object OstreeMutableTree *ret_parent = NULL;
+  glnx_unref_object OstreeMutableTree *ret_parent = NULL;
 
   g_assert (metadata_checksum != NULL);
 
@@ -315,11 +322,9 @@ ostree_mutable_tree_walk (OstreeMutableTree     *self,
                           OstreeMutableTree    **out_subdir,
                           GError               **error)
 {
-  if (start >= split_path->len)
-    {
-      return set_error_noent (error, (char*)split_path->pdata[start]);
-    }
-  else if (start == split_path->len - 1)
+  g_return_val_if_fail (start < split_path->len, FALSE);
+
+  if (start == split_path->len - 1)
     {
       *out_subdir = g_object_ref (self);
       return TRUE;
