@@ -113,9 +113,9 @@ _ostree_repo_file_new_root (OstreeRepo *repo,
 
   g_return_val_if_fail (repo != NULL, NULL);
   g_return_val_if_fail (contents_checksum != NULL, NULL);
-  g_return_val_if_fail (strlen (contents_checksum) == 64, NULL);
+  g_return_val_if_fail (strlen (contents_checksum) == OSTREE_SHA256_STRING_LEN, NULL);
   g_return_val_if_fail (metadata_checksum != NULL, NULL);
-  g_return_val_if_fail (strlen (metadata_checksum) == 64, NULL);
+  g_return_val_if_fail (strlen (metadata_checksum) == OSTREE_SHA256_STRING_LEN, NULL);
 
   self = g_object_new (OSTREE_TYPE_REPO_FILE, NULL);
   self->repo = g_object_ref (repo);
@@ -149,15 +149,15 @@ _ostree_repo_file_new_for_commit (OstreeRepo  *repo,
                                   GError     **error)
 {
   OstreeRepoFile *ret = NULL;
-  gs_unref_variant GVariant *commit_v = NULL;
-  gs_unref_variant GVariant *tree_contents_csum_v = NULL;
-  gs_unref_variant GVariant *tree_metadata_csum_v = NULL;
-  char tree_contents_csum[65];
-  char tree_metadata_csum[65];
+  g_autoptr(GVariant) commit_v = NULL;
+  g_autoptr(GVariant) tree_contents_csum_v = NULL;
+  g_autoptr(GVariant) tree_metadata_csum_v = NULL;
+  char tree_contents_csum[OSTREE_SHA256_STRING_LEN + 1];
+  char tree_metadata_csum[OSTREE_SHA256_STRING_LEN + 1];
 
   g_return_val_if_fail (repo != NULL, NULL);
   g_return_val_if_fail (commit != NULL, NULL);
-  g_return_val_if_fail (strlen (commit) == 64, NULL);
+  g_return_val_if_fail (strlen (commit) == OSTREE_SHA256_STRING_LEN, NULL);
 
   if (!ostree_repo_load_variant (repo, OSTREE_OBJECT_TYPE_COMMIT,
                                  commit, &commit_v, error))
@@ -183,8 +183,8 @@ do_resolve (OstreeRepoFile  *self,
             GError         **error)
 {
   gboolean ret = FALSE;
-  gs_unref_variant GVariant *root_contents = NULL;
-  gs_unref_variant GVariant *root_metadata = NULL;
+  g_autoptr(GVariant) root_contents = NULL;
+  g_autoptr(GVariant) root_metadata = NULL;
 
   g_assert (self->parent == NULL);
 
@@ -213,16 +213,23 @@ do_resolve_nonroot (OstreeRepoFile     *self,
   gboolean ret = FALSE;
   gboolean is_dir;
   int i;
-  gs_unref_variant GVariant *container = NULL;
-  gs_unref_variant GVariant *tree_contents = NULL;
-  gs_unref_variant GVariant *tree_metadata = NULL;
-  gs_unref_variant GVariant *contents_csum_v = NULL;
-  gs_unref_variant GVariant *metadata_csum_v = NULL;
-  gs_free char *tmp_checksum = NULL;
+  g_autoptr(GVariant) container = NULL;
+  g_autoptr(GVariant) tree_contents = NULL;
+  g_autoptr(GVariant) tree_metadata = NULL;
+  g_autoptr(GVariant) contents_csum_v = NULL;
+  g_autoptr(GVariant) metadata_csum_v = NULL;
+  g_autofree char *tmp_checksum = NULL;
 
   if (!ostree_repo_file_ensure_resolved (self->parent, error))
     goto out;
 
+  if (!self->parent->tree_contents)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_DIRECTORY,
+                   "Not a directory");
+      goto out;
+    }
+
   i = ostree_repo_file_tree_find_child (self->parent, self->name, &is_dir, &container);
 
   if (i < 0)
@@ -305,7 +312,7 @@ ostree_repo_file_get_xattrs (OstreeRepoFile  *self,
                              GError         **error)
 {
   gboolean ret = FALSE;
-  gs_unref_variant GVariant *ret_xattrs = NULL;
+  g_autoptr(GVariant) ret_xattrs = NULL;
 
   if (!ostree_repo_file_ensure_resolved (self, error))
     goto out;
@@ -495,12 +502,13 @@ ostree_repo_file_get_uri (GFile *file)
   const char *path;
   char *uri_path;
   char *ret;
+  OstreeRepoFile *root = ostree_repo_file_get_root (self);
 
   path = gs_file_get_path_cached (file);
   uri_path = g_filename_to_uri (path, NULL, NULL);
   g_assert (g_str_has_prefix (uri_path, "file://"));
   ret = g_strconcat ("ostree://",
-                     self->tree_contents_checksum, "/", self->tree_metadata_checksum,
+                     root->tree_contents_checksum, "/", root->tree_metadata_checksum,
                      uri_path+strlen("file://"),
                      NULL);
   g_free (uri_path);
@@ -714,8 +722,8 @@ query_child_info_dir (OstreeRepo               *repo,
                       GError                  **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFileInfo *ret_info = NULL;
-  gs_unref_variant GVariant *metadata = NULL;
+  g_autoptr(GFileInfo) ret_info = NULL;
+  g_autoptr(GVariant) metadata = NULL;
 
   ret_info = g_file_info_new ();
 
@@ -737,52 +745,6 @@ query_child_info_dir (OstreeRepo               *repo,
   return ret;
 }
 
-static gboolean
-bsearch_in_file_variant (GVariant  *variant,
-                         const char *name,
-                         int        *out_pos)
-{
-  gsize imax, imin;
-  gsize imid;
-  gsize n;
-
-  n = g_variant_n_children (variant);
-  if (n == 0)
-    return FALSE;
-
-  imax = n - 1;
-  imin = 0;
-  while (imax >= imin)
-    {
-      gs_unref_variant GVariant *child = NULL;
-      const char *cur;
-      int cmp;
-
-      imid = (imin + imax) / 2;
-
-      child = g_variant_get_child_value (variant, imid);
-      g_variant_get_child (child, 0, "&s", &cur, NULL);      
-
-      cmp = strcmp (cur, name);
-      if (cmp < 0)
-        imin = imid + 1;
-      else if (cmp > 0)
-        {
-          if (imid == 0)
-            break;
-          imax = imid - 1;
-        }
-      else
-        {
-          *out_pos = imid;
-          return TRUE;
-        }
-    }
-
-  *out_pos = imid;
-  return FALSE;
-}
-
 int
 ostree_repo_file_tree_find_child  (OstreeRepoFile  *self,
                                     const char      *name,
@@ -798,7 +760,7 @@ ostree_repo_file_tree_find_child  (OstreeRepoFile  *self,
   dirs_variant = g_variant_get_child_value (self->tree_contents, 1);
 
   i = -1;
-  if (bsearch_in_file_variant (files_variant, name, &i))
+  if (ot_variant_bsearch_str (files_variant, name, &i))
     {
       *is_dir = FALSE;
       ret_container = files_variant;
@@ -806,7 +768,7 @@ ostree_repo_file_tree_find_child  (OstreeRepoFile  *self,
     }
   else
     {
-      if (bsearch_in_file_variant (dirs_variant, name, &i))
+      if (ot_variant_bsearch_str (dirs_variant, name, &i))
         {
           *is_dir = TRUE;
           ret_container = dirs_variant;
@@ -840,12 +802,12 @@ ostree_repo_file_tree_query_child (OstreeRepoFile  *self,
   gboolean ret = FALSE;
   const char *name = NULL;
   int c;
-  gs_unref_object GFileInfo *ret_info = NULL;
-  gs_unref_variant GVariant *files_variant = NULL;
-  gs_unref_variant GVariant *dirs_variant = NULL;
-  gs_unref_variant GVariant *content_csum_v = NULL;
-  gs_unref_variant GVariant *meta_csum_v = NULL;
-  char tmp_checksum[65];
+  g_autoptr(GFileInfo) ret_info = NULL;
+  g_autoptr(GVariant) files_variant = NULL;
+  g_autoptr(GVariant) dirs_variant = NULL;
+  g_autoptr(GVariant) content_csum_v = NULL;
+  g_autoptr(GVariant) meta_csum_v = NULL;
+  char tmp_checksum[OSTREE_SHA256_STRING_LEN+1];
   GFileAttributeMatcher *matcher = NULL;
 
   if (!ostree_repo_file_ensure_resolved (self, error))
@@ -861,9 +823,14 @@ ostree_repo_file_tree_query_child (OstreeRepoFile  *self,
   c = g_variant_n_children (files_variant);
   if (n < c)
     {
+      const guchar *csum_bytes;
+
       g_variant_get_child (files_variant, n, "(&s@ay)", &name, &content_csum_v);
-      ostree_checksum_inplace_from_bytes (ostree_checksum_bytes_peek (content_csum_v),
-                                          tmp_checksum);
+      csum_bytes = ostree_checksum_bytes_peek_validate (content_csum_v, error);
+      if (csum_bytes == NULL)
+        goto out;
+
+      ostree_checksum_inplace_from_bytes (csum_bytes, tmp_checksum);
 
       if (!ostree_repo_load_file (self->repo, tmp_checksum, NULL, &ret_info, NULL,
                                   cancellable, error))
@@ -877,10 +844,15 @@ ostree_repo_file_tree_query_child (OstreeRepoFile  *self,
 
       if (n < c)
         {
+          const guchar *csum_bytes;
+
           g_variant_get_child (dirs_variant, n, "(&s@ay@ay)",
                                &name, NULL, &meta_csum_v);
-          ostree_checksum_inplace_from_bytes (ostree_checksum_bytes_peek (meta_csum_v),
-                                              tmp_checksum);
+          csum_bytes = ostree_checksum_bytes_peek_validate (meta_csum_v, error);
+          if (csum_bytes == NULL)
+            goto out;
+
+          ostree_checksum_inplace_from_bytes (csum_bytes, tmp_checksum);
 
           if (!query_child_info_dir (self->repo, tmp_checksum,
                                      matcher, flags, &ret_info,
@@ -920,7 +892,7 @@ ostree_repo_file_query_info (GFile                *file,
 {
   gboolean ret = FALSE;
   OstreeRepoFile *self = OSTREE_REPO_FILE (file);
-  gs_unref_object GFileInfo *info = NULL;
+  g_autoptr(GFileInfo) info = NULL;
 
   if (!ostree_repo_file_ensure_resolved (self, error))
     goto out;
@@ -972,7 +944,7 @@ ostree_repo_file_read (GFile         *file,
   gboolean ret = FALSE;
   OstreeRepoFile *self = OSTREE_REPO_FILE (file);
   const char *checksum;
-  gs_unref_object GInputStream *ret_stream = NULL;
+  g_autoptr(GInputStream) ret_stream = NULL;
 
   if (!ostree_repo_file_ensure_resolved (self, error))
     goto out;
