@@ -300,7 +300,6 @@ aic_apply_modifier_filter (OstreeRepoArchiveImportContext *ctx,
                            const char  *relpath,
                            GFileInfo  **out_file_info)
 {
-  g_autofree char *hardlink = aic_get_final_entry_hardlink (ctx);
   g_autoptr(GFileInfo) file_info = NULL;
   g_autofree char *abspath = NULL;
   const char *cb_path = NULL;
@@ -914,11 +913,9 @@ ostree_repo_write_archive_to_mtree (OstreeRepo                *self,
 {
 #ifdef HAVE_LIBARCHIVE
   gboolean ret = FALSE;
-  struct archive *a = NULL;
-  g_autoptr(GFileInfo) tmp_dir_info = NULL;
+  ot_cleanup_read_archive struct archive *a = archive_read_new ();
   OstreeRepoImportArchiveOptions opts = { 0, };
 
-  a = archive_read_new ();
 #ifdef HAVE_ARCHIVE_READ_SUPPORT_FILTER_ALL
   archive_read_support_filter_all (a);
 #else
@@ -945,7 +942,9 @@ ostree_repo_write_archive_to_mtree (OstreeRepo                *self,
   ret = TRUE;
  out:
   if (a)
-    (void)archive_read_close (a);
+    {
+      (void)archive_read_close (a);
+    }
   return ret;
 #else
   g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
