@@ -32,6 +32,7 @@
 #include "ostree-diff.h"
 #include "ostree-rollsum.h"
 #include "otutil.h"
+#include "libglnx.h"
 #include "ostree-varint.h"
 #include "bsdiff/bsdiff.h"
 
@@ -131,7 +132,7 @@ xattr_chunk_hash (const void *vp)
     {
       const guint8* name;
       const guint8* value_data;
-      GVariant *value = NULL;
+      g_autoptr(GVariant) value = NULL;
       gsize value_len;
 
       g_variant_get_child (v, i, "(^&ay@ay)",
@@ -156,6 +157,9 @@ xattr_chunk_equals (const void *one, const void *two)
   if (l1 != l2)
     return FALSE;
 
+  if (l1 == 0)
+    return l2 == 0;
+
   return memcmp (g_variant_get_data (v1), g_variant_get_data (v2), l1) == 0;
 }
 
@@ -413,15 +417,11 @@ process_one_object (OstreeRepo                       *repo,
 
 typedef struct {
   char *from_checksum;
-  GBytes *tmp_from;
-  GBytes *tmp_to;
 } ContentBsdiff;
 
 typedef struct {
   char *from_checksum;
   OstreeRollsumMatches *matches;
-  GBytes *tmp_from;
-  GBytes *tmp_to;
 } ContentRollsum;
 
 static void
@@ -429,8 +429,6 @@ content_rollsums_free (ContentRollsum  *rollsum)
 {
   g_free (rollsum->from_checksum);
   _ostree_rollsum_matches_free (rollsum->matches);
-  g_clear_pointer (&rollsum->tmp_from, g_bytes_unref);
-  g_clear_pointer (&rollsum->tmp_to, g_bytes_unref);
   g_free (rollsum);
 }
 
@@ -438,8 +436,6 @@ static void
 content_bsdiffs_free (ContentBsdiff  *bsdiff)
 {
   g_free (bsdiff->from_checksum);
-  g_clear_pointer (&bsdiff->tmp_from, g_bytes_unref);
-  g_clear_pointer (&bsdiff->tmp_to, g_bytes_unref);
   g_free (bsdiff);
 }
 
@@ -450,50 +446,40 @@ static gboolean
 get_unpacked_unlinked_content (OstreeRepo       *repo,
                                const char       *checksum,
                                GBytes          **out_content,
-                               GFileInfo       **out_finfo,
                                GCancellable     *cancellable,
                                GError          **error)
 {
-  gboolean ret = FALSE;
-  g_autofree char *tmpname = g_strdup ("/var/tmp/tmpostree-deltaobj-XXXXXX");
+  g_autofree char *tmpname = NULL;
   glnx_fd_close int fd = -1;
   g_autoptr(GBytes) ret_content = NULL;
   g_autoptr(GInputStream) istream = NULL;
-  g_autoptr(GFileInfo) ret_finfo = NULL;
   g_autoptr(GOutputStream) out = NULL;
 
-  fd = g_mkstemp (tmpname);
-  if (fd == -1)
-    {
-      glnx_set_error_from_errno (error);
-      goto out;
-    }
-  /* Doesn't need a name */
-  (void) unlink (tmpname);
+  if (!glnx_open_tmpfile_linkable_at (AT_FDCWD, "/tmp", O_RDWR | O_CLOEXEC,
+                                      &fd, &tmpname, error))
+    return FALSE;
+  /* We don't need the file name */
+  if (tmpname)
+    (void) unlinkat (AT_FDCWD, tmpname, 0);
 
-  if (!ostree_repo_load_file (repo, checksum, &istream, &ret_finfo, NULL,
+  if (!ostree_repo_load_file (repo, checksum, &istream, NULL, NULL,
                               cancellable, error))
-    goto out;
+    return FALSE;
 
-  g_assert (g_file_info_get_file_type (ret_finfo) == G_FILE_TYPE_REGULAR);
-  
   out = g_unix_output_stream_new (fd, FALSE);
   if (g_output_stream_splice (out, istream, G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
                               cancellable, error) < 0)
-    goto out;
-  
-  { GMappedFile *mfile = g_mapped_file_new_from_fd (fd, FALSE, error);
+    return FALSE;
+
+  { g_autoptr(GMappedFile) mfile = g_mapped_file_new_from_fd (fd, FALSE, error);
     if (!mfile)
-      goto out;
+      return FALSE;
     ret_content = g_mapped_file_get_bytes (mfile);
-    g_mapped_file_unref (mfile);
   }
 
-  ret = TRUE;
   if (out_content)
     *out_content = g_steal_pointer (&ret_content);
- out:
-  return ret;
+  return TRUE;
 }
 
 static gboolean
@@ -506,24 +492,20 @@ try_content_bsdiff (OstreeRepo                       *repo,
                     GError                           **error)
 {
   gboolean ret = FALSE;
-  g_autoptr(GHashTable) from_bsdiff = NULL;
-  g_autoptr(GHashTable) to_bsdiff = NULL;
-  g_autoptr(GBytes) tmp_from = NULL;
-  g_autoptr(GBytes) tmp_to = NULL;
   g_autoptr(GFileInfo) from_finfo = NULL;
   g_autoptr(GFileInfo) to_finfo = NULL;
   ContentBsdiff *ret_bsdiff = NULL;
 
   *out_bsdiff = NULL;
 
-  if (!get_unpacked_unlinked_content (repo, from, &tmp_from, &from_finfo,
-                                      cancellable, error))
-    goto out;
-  if (!get_unpacked_unlinked_content (repo, to, &tmp_to, &to_finfo,
-                                      cancellable, error))
-    goto out;
+  if (!ostree_repo_load_file (repo, from, NULL, &from_finfo, NULL,
+                              cancellable, error))
+    return FALSE;
+  if (!ostree_repo_load_file (repo, to, NULL, &to_finfo, NULL,
+                              cancellable, error))
+    return FALSE;
 
-  if (g_bytes_get_size (tmp_to) + g_bytes_get_size (tmp_from) > max_bsdiff_size_bytes)
+  if (g_file_info_get_size (to_finfo) + g_file_info_get_size (from_finfo) > max_bsdiff_size_bytes)
     {
       ret = TRUE;
       goto out;
@@ -531,8 +513,6 @@ try_content_bsdiff (OstreeRepo                       *repo,
 
   ret_bsdiff = g_new0 (ContentBsdiff, 1);
   ret_bsdiff->from_checksum = g_strdup (from);
-  ret_bsdiff->tmp_from = tmp_from; tmp_from = NULL;
-  ret_bsdiff->tmp_to = tmp_to; tmp_to = NULL;
 
   ret = TRUE;
   if (out_bsdiff)
@@ -551,12 +531,8 @@ try_content_rollsum (OstreeRepo                       *repo,
                      GError                          **error)
 {
   gboolean ret = FALSE;
-  g_autoptr(GHashTable) from_rollsum = NULL;
-  g_autoptr(GHashTable) to_rollsum = NULL;
   g_autoptr(GBytes) tmp_from = NULL;
   g_autoptr(GBytes) tmp_to = NULL;
-  g_autoptr(GFileInfo) from_finfo = NULL;
-  g_autoptr(GFileInfo) to_finfo = NULL;
   OstreeRollsumMatches *matches = NULL;
   ContentRollsum *ret_rollsum = NULL;
 
@@ -565,11 +541,9 @@ try_content_rollsum (OstreeRepo                       *repo,
   /* Load the content objects, splice them to uncompressed temporary files that
    * we can just mmap() and seek around in conveniently.
    */
-  if (!get_unpacked_unlinked_content (repo, from, &tmp_from, &from_finfo,
-                                      cancellable, error))
+  if (!get_unpacked_unlinked_content (repo, from, &tmp_from, cancellable, error))
     goto out;
-  if (!get_unpacked_unlinked_content (repo, to, &tmp_to, &to_finfo,
-                                      cancellable, error))
+  if (!get_unpacked_unlinked_content (repo, to, &tmp_to, cancellable, error))
     goto out;
 
   matches = _ostree_compute_rollsum_matches (tmp_from, tmp_to);
@@ -596,10 +570,8 @@ try_content_rollsum (OstreeRepo                       *repo,
 
   ret_rollsum = g_new0 (ContentRollsum, 1);
   ret_rollsum->from_checksum = g_strdup (from);
-  ret_rollsum->matches = matches; matches = NULL;
-  ret_rollsum->tmp_from = tmp_from; tmp_from = NULL;
-  ret_rollsum->tmp_to = tmp_to; tmp_to = NULL;
-  
+  ret_rollsum->matches = g_steal_pointer (&matches);
+
   ret = TRUE;
   if (out_rollsum)
     *out_rollsum = g_steal_pointer (&ret_rollsum);
@@ -655,7 +627,7 @@ process_one_rollsum (OstreeRepo                       *repo,
 {
   gboolean ret = FALSE;
   guint64 content_size;
-  g_autoptr(GInputStream) content_stream = NULL;
+  g_autoptr(GBytes) tmp_to = NULL;
   g_autoptr(GFileInfo) content_finfo = NULL;
   g_autoptr(GVariant) content_xattrs = NULL;
   OstreeStaticDeltaPartBuilder *current_part = *current_part_val;
@@ -669,9 +641,13 @@ process_one_rollsum (OstreeRepo                       *repo,
       *current_part_val = current_part = allocate_part (builder);
     }
 
-  tmp_to_buf = g_bytes_get_data (rollsum->tmp_to, &tmp_to_len);
+  if (!get_unpacked_unlinked_content (repo, to_checksum, &tmp_to,
+                                      cancellable, error))
+    goto out;
 
-  if (!ostree_repo_load_file (repo, to_checksum, &content_stream,
+  tmp_to_buf = g_bytes_get_data (tmp_to, &tmp_to_len);
+
+  if (!ostree_repo_load_file (repo, to_checksum, NULL,
                               &content_finfo, &content_xattrs,
                               cancellable, error))
     goto out;
@@ -758,9 +734,6 @@ process_one_rollsum (OstreeRepo                       *repo,
     g_string_append_c (current_part->operations, (gchar)OSTREE_STATIC_DELTA_OP_CLOSE);
   }
 
-  g_clear_pointer (&rollsum->tmp_from, g_bytes_unref);
-  g_clear_pointer (&rollsum->tmp_to, g_bytes_unref);
-
   ret = TRUE;
  out:
   return ret;
@@ -777,10 +750,11 @@ process_one_bsdiff (OstreeRepo                       *repo,
 {
   gboolean ret = FALSE;
   guint64 content_size;
-  g_autoptr(GInputStream) content_stream = NULL;
   g_autoptr(GFileInfo) content_finfo = NULL;
   g_autoptr(GVariant) content_xattrs = NULL;
   OstreeStaticDeltaPartBuilder *current_part = *current_part_val;
+  g_autoptr(GBytes) tmp_from = NULL;
+  g_autoptr(GBytes) tmp_to = NULL;
   const guint8 *tmp_to_buf;
   gsize tmp_to_len;
   const guint8 *tmp_from_buf;
@@ -793,10 +767,17 @@ process_one_bsdiff (OstreeRepo                       *repo,
       *current_part_val = current_part = allocate_part (builder);
     }
 
-  tmp_to_buf = g_bytes_get_data (bsdiff_content->tmp_to, &tmp_to_len);
-  tmp_from_buf = g_bytes_get_data (bsdiff_content->tmp_from, &tmp_from_len);
+  if (!get_unpacked_unlinked_content (repo, bsdiff_content->from_checksum, &tmp_from,
+                                      cancellable, error))
+    goto out;
+  if (!get_unpacked_unlinked_content (repo, to_checksum, &tmp_to,
+                                      cancellable, error))
+    goto out;
 
-  if (!ostree_repo_load_file (repo, to_checksum, &content_stream,
+  tmp_to_buf = g_bytes_get_data (tmp_to, &tmp_to_len);
+  tmp_from_buf = g_bytes_get_data (tmp_from, &tmp_from_len);
+
+  if (!ostree_repo_load_file (repo, to_checksum, NULL,
                               &content_finfo, &content_xattrs,
                               cancellable, error))
     goto out;
@@ -857,9 +838,6 @@ process_one_bsdiff (OstreeRepo                       *repo,
 
   g_string_append_c (current_part->operations, (gchar)OSTREE_STATIC_DELTA_OP_UNSET_READ_SOURCE);
 
-  g_clear_pointer (&bsdiff_content->tmp_from, g_bytes_unref);
-  g_clear_pointer (&bsdiff_content->tmp_to, g_bytes_unref);
-
   ret = TRUE;
  out:
   return ret;
@@ -884,14 +862,12 @@ generate_delta_lowlatency (OstreeRepo                       *repo,
   g_autoptr(GVariant) to_commit = NULL;
   g_autoptr(GHashTable) to_reachable_objects = NULL;
   g_autoptr(GHashTable) from_reachable_objects = NULL;
-  g_autoptr(GHashTable) from_regfile_content = NULL;
   g_autoptr(GHashTable) new_reachable_metadata = NULL;
   g_autoptr(GHashTable) new_reachable_regfile_content = NULL;
   g_autoptr(GHashTable) new_reachable_symlink_content = NULL;
   g_autoptr(GHashTable) modified_regfile_content = NULL;
   g_autoptr(GHashTable) rollsum_optimized_content_objects = NULL;
   g_autoptr(GHashTable) bsdiff_optimized_content_objects = NULL;
-  g_autoptr(GHashTable) content_object_to_size = NULL;
 
   if (from != NULL)
     {
@@ -935,9 +911,8 @@ generate_delta_lowlatency (OstreeRepo                       *repo,
 
       ostree_object_name_deserialize (serialized_key, &checksum, &objtype);
 
-      g_variant_ref (serialized_key);
       if (OSTREE_OBJECT_TYPE_IS_META (objtype))
-        g_hash_table_add (new_reachable_metadata, serialized_key);
+        g_hash_table_add (new_reachable_metadata, g_variant_ref (serialized_key));
       else
         {
           g_autoptr(GFileInfo) finfo = NULL;
@@ -979,8 +954,9 @@ generate_delta_lowlatency (OstreeRepo                       *repo,
     }
 
   /* We already ship the to commit in the superblock, don't ship it twice */
-  g_hash_table_remove (new_reachable_metadata,
-                       ostree_object_name_serialize (to, OSTREE_OBJECT_TYPE_COMMIT));
+  { g_autoptr(GVariant) commit = ostree_object_name_serialize (to, OSTREE_OBJECT_TYPE_COMMIT);
+    g_hash_table_remove (new_reachable_metadata, commit);
+  }
 
   rollsum_optimized_content_objects = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                              g_free,
@@ -1261,7 +1237,7 @@ ostree_repo_static_delta_generate (OstreeRepo                   *self,
   guint min_fallback_size;
   guint max_bsdiff_size;
   guint max_chunk_size;
-  g_auto(GVariantBuilder) metadata_builder = {{0,}};
+  g_auto(GVariantBuilder) metadata_builder = OT_VARIANT_BUILDER_INITIALIZER;
   DeltaOpts delta_opts = DELTAOPT_FLAG_NONE;
   guint64 total_compressed_size = 0;
   guint64 total_uncompressed_size = 0;
@@ -1273,7 +1249,6 @@ ostree_repo_static_delta_generate (OstreeRepo                   *self,
   const char *opt_filename;
   g_autofree char *descriptor_name = NULL;
   glnx_fd_close int descriptor_dfd = -1;
-  g_autoptr(GVariant) tmp_metadata = NULL;
   g_autoptr(GVariant) fallback_headers = NULL;
   g_autoptr(GVariant) detached = NULL;
   gboolean inline_parts;
@@ -1384,10 +1359,9 @@ ostree_repo_static_delta_generate (OstreeRepo                   *self,
   for (i = 0; i < builder.parts->len; i++)
     {
       OstreeStaticDeltaPartBuilder *part_builder = builder.parts->pdata[i];
-      GBytes *payload_b;
-      GBytes *operations_b;
+      g_autoptr(GBytes) payload_b;
+      g_autoptr(GBytes) operations_b;
       g_autofree guchar *part_checksum = NULL;
-      g_autoptr(GChecksum) checksum = NULL;
       g_autoptr(GBytes) objtype_checksum_array = NULL;
       g_autoptr(GBytes) checksum_bytes = NULL;
       g_autoptr(GOutputStream) part_temp_outstream = NULL;
@@ -1399,8 +1373,8 @@ ostree_repo_static_delta_generate (OstreeRepo                   *self,
       g_autoptr(GVariant) delta_part_content = NULL;
       g_autoptr(GVariant) delta_part = NULL;
       g_autoptr(GVariant) delta_part_header = NULL;
-      g_auto(GVariantBuilder) mode_builder = {{0,}};
-      g_auto(GVariantBuilder) xattr_builder = {{0,}};
+      g_auto(GVariantBuilder) mode_builder = OT_VARIANT_BUILDER_INITIALIZER;
+      g_auto(GVariantBuilder) xattr_builder = OT_VARIANT_BUILDER_INITIALIZER;
       guint8 compression_type_char;
 
       g_variant_builder_init (&mode_builder, G_VARIANT_TYPE ("a(uuu)"));
@@ -1441,10 +1415,11 @@ ostree_repo_static_delta_generate (OstreeRepo                   *self,
       }
 
       /* FIXME - avoid duplicating memory here */
-      delta_part = g_variant_new ("(y@ay)",
-                                  compression_type_char,
-                                  ot_gvariant_new_ay_bytes (g_memory_output_stream_steal_as_bytes (part_payload_out)));
-      g_variant_ref_sink (delta_part);
+      { g_autoptr(GBytes) payload = g_memory_output_stream_steal_as_bytes (part_payload_out);
+        delta_part = g_variant_ref_sink (g_variant_new ("(y@ay)",
+                                                        compression_type_char,
+                                                        ot_gvariant_new_ay_bytes (payload)));
+      }
 
       if (inline_parts)
         {
@@ -1558,17 +1533,17 @@ ostree_repo_static_delta_generate (OstreeRepo                   *self,
     /* floating */ GVariant *to_csum_v =
       ostree_checksum_to_bytes_v (to);
 
-    delta_descriptor = g_variant_new ("(@a{sv}t@ay@ay@" OSTREE_COMMIT_GVARIANT_STRING "ay"
-                                      "a" OSTREE_STATIC_DELTA_META_ENTRY_FORMAT
-                                      "@a" OSTREE_STATIC_DELTA_FALLBACK_FORMAT ")",
-                                      g_variant_builder_end (&metadata_builder),
-                                      GUINT64_TO_BE (g_date_time_to_unix (now)),
-                                      from_csum_v,
-                                      to_csum_v,
-                                      to_commit,
-                                      g_variant_builder_new (G_VARIANT_TYPE ("ay")),
-                                      part_headers,
-                                      fallback_headers);
+    delta_descriptor = g_variant_ref_sink (g_variant_new ("(@a{sv}t@ay@ay@" OSTREE_COMMIT_GVARIANT_STRING "@ay"
+                                                          "a" OSTREE_STATIC_DELTA_META_ENTRY_FORMAT
+                                                          "@a" OSTREE_STATIC_DELTA_FALLBACK_FORMAT ")",
+                                                          g_variant_builder_end (&metadata_builder),
+                                                          GUINT64_TO_BE (g_date_time_to_unix (now)),
+                                                          from_csum_v,
+                                                          to_csum_v,
+                                                          to_commit,
+                                                          ot_gvariant_new_bytearray ((guchar*)"", 0),
+                                                          part_headers,
+                                                          fallback_headers));
     g_date_time_unref (now);
   }
 
