@@ -21,7 +21,7 @@
 #include "config.h"
 
 #include "otutil.h"
-#include "libgsystem.h"
+#include "ostree-linuxfsutil.h"
 
 #include "ostree-sysroot-private.h"
 
@@ -32,9 +32,9 @@ _ostree_sysroot_list_deployment_dirs_for_os (GFile               *osdir,
                                              GError             **error)
 {
   gboolean ret = FALSE;
-  const char *osname = gs_file_get_basename_cached (osdir);
-  gs_unref_object GFileEnumerator *dir_enum = NULL;
-  gs_unref_object GFile *osdeploy_dir = NULL;
+  const char *osname = glnx_basename (gs_file_get_path_cached (osdir));
+  g_autoptr(GFileEnumerator) dir_enum = NULL;
+  g_autoptr(GFile) osdeploy_dir = NULL;
   GError *temp_error = NULL;
 
   osdeploy_dir = g_file_get_child (osdir, "deploy");
@@ -61,12 +61,12 @@ _ostree_sysroot_list_deployment_dirs_for_os (GFile               *osdir,
       const char *name;
       GFileInfo *file_info = NULL;
       GFile *child = NULL;
-      gs_unref_object OstreeDeployment *deployment = NULL;
-      gs_free char *csum = NULL;
+      glnx_unref_object OstreeDeployment *deployment = NULL;
+      g_autofree char *csum = NULL;
       gint deployserial;
 
-      if (!gs_file_enumerator_iterate (dir_enum, &file_info, &child,
-                                       cancellable, error))
+      if (!g_file_enumerator_iterate (dir_enum, &file_info, &child,
+                                      cancellable, error))
         goto out;
       if (file_info == NULL)
         break;
@@ -96,9 +96,9 @@ list_all_deployment_directories (OstreeSysroot       *self,
                                  GError             **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFileEnumerator *dir_enum = NULL;
-  gs_unref_object GFile *deploydir = NULL;
-  gs_unref_ptrarray GPtrArray *ret_deployments = NULL;
+  g_autoptr(GFileEnumerator) dir_enum = NULL;
+  g_autoptr(GFile) deploydir = NULL;
+  g_autoptr(GPtrArray) ret_deployments = NULL;
   GError *temp_error = NULL;
 
   deploydir = g_file_resolve_relative_path (self->path, "ostree/deploy");
@@ -127,8 +127,8 @@ list_all_deployment_directories (OstreeSysroot       *self,
       GFileInfo *file_info = NULL;
       GFile *child = NULL;
 
-      if (!gs_file_enumerator_iterate (dir_enum, &file_info, &child,
-                                       NULL, error))
+      if (!g_file_enumerator_iterate (dir_enum, &file_info, &child,
+                                      NULL, error))
         goto out;
       if (file_info == NULL)
         break;
@@ -183,9 +183,9 @@ list_all_boot_directories (OstreeSysroot       *self,
                            GError             **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFileEnumerator *dir_enum = NULL;
-  gs_unref_object GFile *boot_ostree = NULL;
-  gs_unref_ptrarray GPtrArray *ret_bootdirs = NULL;
+  g_autoptr(GFileEnumerator) dir_enum = NULL;
+  g_autoptr(GFile) boot_ostree = NULL;
+  g_autoptr(GPtrArray) ret_bootdirs = NULL;
   GError *temp_error = NULL;
 
   boot_ostree = g_file_resolve_relative_path (self->path, "boot/ostree");
@@ -215,8 +215,8 @@ list_all_boot_directories (OstreeSysroot       *self,
       GFile *child = NULL;
       const char *name;
 
-      if (!gs_file_enumerator_iterate (dir_enum, &file_info, &child,
-                                       NULL, error))
+      if (!g_file_enumerator_iterate (dir_enum, &file_info, &child,
+                                      NULL, error))
         goto out;
       if (file_info == NULL)
         break;
@@ -249,34 +249,34 @@ cleanup_other_bootversions (OstreeSysroot       *self,
   gboolean ret = FALSE;
   int cleanup_bootversion;
   int cleanup_subbootversion;
-  gs_unref_object GFile *cleanup_boot_dir = NULL;
+  g_autoptr(GFile) cleanup_boot_dir = NULL;
 
   cleanup_bootversion = self->bootversion == 0 ? 1 : 0;
   cleanup_subbootversion = self->subbootversion == 0 ? 1 : 0;
 
   cleanup_boot_dir = ot_gfile_resolve_path_printf (self->path, "boot/loader.%d", cleanup_bootversion);
-  if (!gs_shutil_rm_rf (cleanup_boot_dir, cancellable, error))
+  if (!glnx_shutil_rm_rf_at (AT_FDCWD, gs_file_get_path_cached (cleanup_boot_dir), cancellable, error))
     goto out;
   g_clear_object (&cleanup_boot_dir);
 
   cleanup_boot_dir = ot_gfile_resolve_path_printf (self->path, "ostree/boot.%d", cleanup_bootversion);
-  if (!gs_shutil_rm_rf (cleanup_boot_dir, cancellable, error))
+  if (!glnx_shutil_rm_rf_at (AT_FDCWD, gs_file_get_path_cached (cleanup_boot_dir), cancellable, error))
     goto out;
   g_clear_object (&cleanup_boot_dir);
 
   cleanup_boot_dir = ot_gfile_resolve_path_printf (self->path, "ostree/boot.%d.0", cleanup_bootversion);
-  if (!gs_shutil_rm_rf (cleanup_boot_dir, cancellable, error))
+  if (!glnx_shutil_rm_rf_at (AT_FDCWD, gs_file_get_path_cached (cleanup_boot_dir), cancellable, error))
     goto out;
   g_clear_object (&cleanup_boot_dir);
 
   cleanup_boot_dir = ot_gfile_resolve_path_printf (self->path, "ostree/boot.%d.1", cleanup_bootversion);
-  if (!gs_shutil_rm_rf (cleanup_boot_dir, cancellable, error))
+  if (!glnx_shutil_rm_rf_at (AT_FDCWD, gs_file_get_path_cached (cleanup_boot_dir), cancellable, error))
     goto out;
   g_clear_object (&cleanup_boot_dir);
 
   cleanup_boot_dir = ot_gfile_resolve_path_printf (self->path, "ostree/boot.%d.%d", self->bootversion,
                                                    cleanup_subbootversion);
-  if (!gs_shutil_rm_rf (cleanup_boot_dir, cancellable, error))
+  if (!glnx_shutil_rm_rf_at (AT_FDCWD, gs_file_get_path_cached (cleanup_boot_dir), cancellable, error))
     goto out;
   g_clear_object (&cleanup_boot_dir);
 
@@ -291,26 +291,26 @@ cleanup_old_deployments (OstreeSysroot       *self,
                          GError             **error)
 {
   gboolean ret = FALSE;
-  guint32 root_device;
-  guint64 root_inode;
+  struct stat root_stbuf;
   guint i;
-  gs_unref_object GFile *active_root = g_file_new_for_path ("/");
-  gs_unref_hashtable GHashTable *active_deployment_dirs = NULL;
-  gs_unref_hashtable GHashTable *active_boot_checksums = NULL;
-  gs_unref_ptrarray GPtrArray *all_deployment_dirs = NULL;
-  gs_unref_ptrarray GPtrArray *all_boot_dirs = NULL;
+  g_autoptr(GHashTable) active_deployment_dirs = NULL;
+  g_autoptr(GHashTable) active_boot_checksums = NULL;
+  g_autoptr(GPtrArray) all_deployment_dirs = NULL;
+  g_autoptr(GPtrArray) all_boot_dirs = NULL;
 
-  if (!_ostree_sysroot_get_devino (active_root, &root_device, &root_inode,
-                                   cancellable, error))
-    goto out;
+  if (stat ("/", &root_stbuf) != 0)
+    {
+      glnx_set_error_from_errno (error);
+      goto out;
+    }
 
-  active_deployment_dirs = g_hash_table_new_full (g_file_hash, (GEqualFunc)g_file_equal, g_object_unref, NULL);
+  active_deployment_dirs = g_hash_table_new_full (g_str_hash, (GEqualFunc)g_str_equal, g_free, NULL);
   active_boot_checksums = g_hash_table_new_full (g_str_hash, (GEqualFunc)g_str_equal, g_free, NULL);
 
   for (i = 0; i < self->deployments->len; i++)
     {
       OstreeDeployment *deployment = self->deployments->pdata[i];
-      GFile *deployment_path = ostree_sysroot_get_deployment_directory (self, deployment);
+      char *deployment_path = ostree_sysroot_get_deployment_dirpath (self, deployment);
       char *bootcsum = g_strdup (ostree_deployment_get_bootcsum (deployment));
       /* Transfer ownership */
       g_hash_table_replace (active_deployment_dirs, deployment_path, deployment_path);
@@ -324,26 +324,38 @@ cleanup_old_deployments (OstreeSysroot       *self,
   for (i = 0; i < all_deployment_dirs->len; i++)
     {
       OstreeDeployment *deployment = all_deployment_dirs->pdata[i];
-      gs_unref_object GFile *deployment_path = ostree_sysroot_get_deployment_directory (self, deployment);
-      gs_unref_object GFile *origin_path = ostree_sysroot_get_deployment_origin_path (deployment_path);
+      g_autofree char *deployment_path = ostree_sysroot_get_deployment_dirpath (self, deployment);
+      g_autofree char *origin_relpath = ostree_deployment_get_origin_relpath (deployment);
+
       if (!g_hash_table_lookup (active_deployment_dirs, deployment_path))
         {
-          guint32 device;
-          guint64 inode;
+          struct stat stbuf;
+          glnx_fd_close int deployment_fd = -1;
 
-          if (!_ostree_sysroot_get_devino (deployment_path, &device, &inode,
-                                           cancellable, error))
+          if (!glnx_opendirat (self->sysroot_fd, deployment_path, TRUE,
+                               &deployment_fd, error))
             goto out;
 
+          if (fstat (deployment_fd, &stbuf) != 0)
+            {
+              glnx_set_error_from_errno (error);
+              goto out;
+            }
+
           /* This shouldn't happen, because higher levels should
            * disallow having the booted deployment not in the active
            * deployment list, but let's be extra safe. */
-          if (device == root_device && inode == root_inode)
+          if (stbuf.st_dev == root_stbuf.st_dev &&
+              stbuf.st_ino == root_stbuf.st_ino)
             continue;
 
-          if (!gs_shutil_rm_rf (deployment_path, cancellable, error))
+          if (!_ostree_linuxfs_fd_alter_immutable_flag (deployment_fd, FALSE,
+                                                        cancellable, error))
             goto out;
-          if (!gs_shutil_rm_rf (origin_path, cancellable, error))
+          
+          if (!glnx_shutil_rm_rf_at (self->sysroot_fd, deployment_path, cancellable, error))
+            goto out;
+          if (!glnx_shutil_rm_rf_at (self->sysroot_fd, origin_relpath, cancellable, error))
             goto out;
         }
     }
@@ -355,17 +367,17 @@ cleanup_old_deployments (OstreeSysroot       *self,
   for (i = 0; i < all_boot_dirs->len; i++)
     {
       GFile *bootdir = all_boot_dirs->pdata[i];
-      gs_free char *osname = NULL;
-      gs_free char *bootcsum = NULL;
+      g_autofree char *osname = NULL;
+      g_autofree char *bootcsum = NULL;
 
-      if (!parse_bootdir_name (gs_file_get_basename_cached (bootdir),
+      if (!parse_bootdir_name (glnx_basename (gs_file_get_path_cached (bootdir)),
                                &osname, &bootcsum))
         g_assert_not_reached ();
 
       if (g_hash_table_lookup (active_boot_checksums, bootcsum))
         continue;
 
-      if (!gs_shutil_rm_rf (bootdir, cancellable, error))
+      if (!glnx_shutil_rm_rf_at (AT_FDCWD, gs_file_get_path_cached (bootdir), cancellable, error))
         goto out;
     }
 
@@ -382,14 +394,14 @@ cleanup_ref_prefix (OstreeRepo         *repo,
                     GError            **error)
 {
   gboolean ret = FALSE;
-  gs_free char *prefix = NULL;
-  gs_unref_hashtable GHashTable *refs = NULL;
+  g_autofree char *prefix = NULL;
+  g_autoptr(GHashTable) refs = NULL;
   GHashTableIter hashiter;
   gpointer hashkey, hashvalue;
 
   prefix = g_strdup_printf ("ostree/%d/%d", bootversion, subbootversion);
 
-  if (!ostree_repo_list_refs (repo, prefix, &refs, cancellable, error))
+  if (!ostree_repo_list_refs_ext (repo, prefix, &refs, OSTREE_REPO_LIST_REFS_EXT_NONE, cancellable, error))
     goto out;
 
   if (!ostree_repo_prepare_transaction (repo, NULL, cancellable, error))
@@ -398,8 +410,7 @@ cleanup_ref_prefix (OstreeRepo         *repo,
   g_hash_table_iter_init (&hashiter, refs);
   while (g_hash_table_iter_next (&hashiter, &hashkey, &hashvalue))
     {
-      const char *suffix = hashkey;
-      gs_free char *ref = g_strconcat (prefix, "/", suffix, NULL);
+      const char *ref = hashkey;
       ostree_repo_transaction_set_refspec (repo, ref, NULL);
     }
 
@@ -413,20 +424,18 @@ cleanup_ref_prefix (OstreeRepo         *repo,
 }
 
 static gboolean
-generate_deployment_refs_and_prune (OstreeSysroot       *self,
-                                    OstreeRepo          *repo,
-                                    int                  bootversion,
-                                    int                  subbootversion,
-                                    GPtrArray           *deployments,
-                                    GCancellable        *cancellable,
-                                    GError             **error)
+generate_deployment_refs (OstreeSysroot       *self,
+                          OstreeRepo          *repo,
+                          int                  bootversion,
+                          int                  subbootversion,
+                          GPtrArray           *deployments,
+                          GCancellable        *cancellable,
+                          GError             **error)
 {
   gboolean ret = FALSE;
   int cleanup_bootversion;
   int cleanup_subbootversion;
   guint i;
-  gint n_objects_total, n_objects_pruned;
-  guint64 freed_space;
 
   cleanup_bootversion = (bootversion == 0) ? 1 : 0;
   cleanup_subbootversion = (subbootversion == 0) ? 1 : 0;
@@ -446,7 +455,7 @@ generate_deployment_refs_and_prune (OstreeSysroot       *self,
   for (i = 0; i < deployments->len; i++)
     {
       OstreeDeployment *deployment = deployments->pdata[i];
-      gs_free char *refname = g_strdup_printf ("ostree/%d/%d/%u",
+      g_autofree char *refname = g_strdup_printf ("ostree/%d/%d/%u",
                                                bootversion, subbootversion,
                                                i);
 
@@ -459,22 +468,39 @@ generate_deployment_refs_and_prune (OstreeSysroot       *self,
         goto out;
     }
 
-  if (!ostree_repo_prune (repo, OSTREE_REPO_PRUNE_FLAGS_REFS_ONLY, 0,
-                          &n_objects_total, &n_objects_pruned, &freed_space,
-                          cancellable, error))
-    goto out;
-  if (freed_space > 0)
-    {
-      char *freed_space_str = g_format_size_full (freed_space, 0);
-      g_print ("Freed objects: %s\n", freed_space_str);
-    }
-
   ret = TRUE;
  out:
   ostree_repo_abort_transaction (repo, cancellable, NULL);
   return ret;
 }
-  
+
+static gboolean
+prune_repo (OstreeRepo    *repo,
+            GCancellable  *cancellable,
+            GError       **error)
+{
+  gint n_objects_total;
+  gint n_objects_pruned;
+  guint64 freed_space;
+  gboolean ret = FALSE;
+
+  if (!ostree_repo_prune (repo, OSTREE_REPO_PRUNE_FLAGS_REFS_ONLY, 0,
+                          &n_objects_total, &n_objects_pruned, &freed_space,
+                          cancellable, error))
+    goto out;
+
+  if (freed_space > 0)
+    {
+      g_autofree char *freed_space_str = g_format_size_full (freed_space, 0);
+      g_print ("Freed objects: %s\n", freed_space_str);
+    }
+
+  ret = TRUE;
+
+out:
+  return ret;
+}
+
 /**
  * ostree_sysroot_cleanup:
  * @self: Sysroot
@@ -489,28 +515,76 @@ ostree_sysroot_cleanup (OstreeSysroot       *self,
                         GCancellable        *cancellable,
                         GError             **error)
 {
-  gboolean ret = FALSE;
-  gs_unref_object OstreeRepo *repo = NULL;
+  OstreeSysrootCleanupFlags flags;
 
+  /* Do everything. */
+  flags = OSTREE_SYSROOT_CLEANUP_ALL;
+
+  return _ostree_sysroot_piecemeal_cleanup (self, flags, cancellable, error);
+}
+
+/**
+ * ostree_sysroot_prepare_cleanup:
+ * @self: Sysroot
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Like ostree_sysroot_cleanup() in that it cleans up incomplete deployments
+ * and old boot versions, but does NOT prune the repository.
+ */
+gboolean
+ostree_sysroot_prepare_cleanup (OstreeSysroot  *self,
+                                GCancellable   *cancellable,
+                                GError        **error)
+{
+  OstreeSysrootCleanupFlags flags;
+
+  /* Do everything EXCEPT pruning the repository. */
+  flags = OSTREE_SYSROOT_CLEANUP_ALL & ~OSTREE_SYSROOT_CLEANUP_PRUNE_REPO;
+
+  return _ostree_sysroot_piecemeal_cleanup (self, flags, cancellable, error);
+}
+
+gboolean
+_ostree_sysroot_piecemeal_cleanup (OstreeSysroot              *self,
+                                   OstreeSysrootCleanupFlags   flags,
+                                   GCancellable               *cancellable,
+                                   GError                    **error)
+{
+  gboolean ret = FALSE;
+  glnx_unref_object OstreeRepo *repo = NULL;
+
+  g_return_val_if_fail (OSTREE_IS_SYSROOT (self), FALSE);
   g_return_val_if_fail (self->loaded, FALSE);
 
-  if (!cleanup_other_bootversions (self, cancellable, error))
-    goto out;
-
-  if (!cleanup_old_deployments (self, cancellable, error))
-    goto out;
-
-  if (self->deployments->len > 0)
+  if (flags & OSTREE_SYSROOT_CLEANUP_BOOTVERSIONS)
     {
-      if (!ostree_sysroot_get_repo (self, &repo, cancellable, error))
+      if (!cleanup_other_bootversions (self, cancellable, error))
         goto out;
+    }
 
-      if (!generate_deployment_refs_and_prune (self, repo, self->bootversion,
-                                               self->subbootversion, self->deployments,
-                                               cancellable, error))
+  if (flags & OSTREE_SYSROOT_CLEANUP_DEPLOYMENTS)
+    {
+      if (!cleanup_old_deployments (self, cancellable, error))
         goto out;
     }
 
+  if (!ostree_sysroot_get_repo (self, &repo, cancellable, error))
+    goto out;
+  
+  if (!generate_deployment_refs (self, repo,
+                                 self->bootversion,
+                                 self->subbootversion,
+                                 self->deployments,
+                                 cancellable, error))
+    goto out;
+  
+  if (flags & OSTREE_SYSROOT_CLEANUP_PRUNE_REPO)
+    {
+      if (!prune_repo (repo, cancellable, error))
+            goto out;
+    }
+
   ret = TRUE;
  out:
   return ret;
