@@ -77,7 +77,9 @@ parse_refspec (OstreeSysrootUpgrader  *self,
 
   if ((self->flags & OSTREE_SYSROOT_UPGRADER_FLAGS_IGNORE_UNCONFIGURED) == 0)
     {
-      /* If explicit action by the OS creator is requried to upgrade, print their text as an error */
+      /* If explicit action by the OS creator is requried to upgrade, print their text as an error.
+       * NOTE: If changing this, see the matching implementation in ostree-repo-pull.c.
+       */
       unconfigured_state = g_key_file_get_string (self->origin, "origin", "unconfigured-state", NULL);
       if (unconfigured_state)
         {
@@ -105,6 +107,7 @@ parse_refspec (OstreeSysrootUpgrader  *self,
   csum = g_key_file_get_string (self->origin, "origin", "override-commit", NULL);
   if (csum != NULL && !ostree_validate_checksum_string (csum, error))
     goto out;
+  g_clear_pointer (&self->override_csum, g_free);
   self->override_csum = g_steal_pointer (&csum);
 
   ret = TRUE;
@@ -188,6 +191,7 @@ ostree_sysroot_upgrader_finalize (GObject *object)
   g_free (self->origin_remote);
   g_free (self->origin_ref);
   g_free (self->override_csum);
+  g_free (self->new_revision);
 
   G_OBJECT_CLASS (ostree_sysroot_upgrader_parent_class)->finalize (object);
 }
@@ -530,7 +534,6 @@ ostree_sysroot_upgrader_pull_one_dir (OstreeSysrootUpgrader  *self,
   glnx_unref_object OstreeRepo *repo = NULL;
   char *refs_to_fetch[] = { NULL, NULL };
   const char *from_revision = NULL;
-  g_autofree char *new_revision = NULL;
   g_autofree char *origin_refspec = NULL;
 
   if (self->override_csum != NULL)
