@@ -92,7 +92,7 @@ ot_gio_splice_update_checksum (GOutputStream  *out,
         }
       while (bytes_read > 0);
     }
-  else
+  else if (out != NULL)
     {
       if (g_output_stream_splice (out, in, 0, cancellable, error) < 0)
         goto out;
@@ -112,7 +112,7 @@ ot_gio_splice_get_checksum (GOutputStream  *out,
 {
   gboolean ret = FALSE;
   GChecksum *checksum = NULL;
-  gs_free guchar *ret_csum = NULL;
+  g_autofree guchar *ret_csum = NULL;
 
   checksum = g_checksum_new (G_CHECKSUM_SHA256);
 
@@ -139,6 +139,32 @@ ot_gio_checksum_stream (GInputStream   *in,
   return ot_gio_splice_get_checksum (NULL, in, out_csum, cancellable, error);
 }
 
+char *
+ot_checksum_file_at (int             dfd,
+                     const char     *path,
+                     GChecksumType   checksum_type,
+                     GCancellable   *cancellable,
+                     GError        **error)
+{
+  GChecksum *checksum = NULL;
+  char *ret = NULL;
+  g_autoptr(GInputStream) in = NULL;
+
+  if (!ot_openat_read_stream (dfd, path, TRUE, &in, cancellable, error))
+    goto out;
+
+  checksum = g_checksum_new (checksum_type);
+
+  if (!ot_gio_splice_update_checksum (NULL, in, checksum, cancellable, error))
+    goto out;
+
+  ret = g_strdup (g_checksum_get_string (checksum));
+ out:
+  g_clear_pointer (&checksum, (GDestroyNotify) g_checksum_free);
+  return ret;
+
+}
+
 static void
 checksum_stream_thread (GSimpleAsyncResult   *result,
                         GObject              *object,
@@ -181,4 +207,3 @@ ot_gio_checksum_stream_finish (GInputStream   *in,
   g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == ot_gio_checksum_stream_async);
   return g_memdup (g_simple_async_result_get_op_res_gpointer (simple), 32);
 }
-
