@@ -38,6 +38,10 @@ ot_keyfile_get_boolean_with_default (GKeyFile      *keyfile,
   GError *temp_error = NULL;
   gboolean ret_bool;
 
+  g_return_val_if_fail (keyfile != NULL, ret);
+  g_return_val_if_fail (section != NULL, ret);
+  g_return_val_if_fail (value != NULL, ret);
+
   ret_bool = g_key_file_get_boolean (keyfile, section, value, &temp_error);
   if (temp_error)
     {
@@ -69,7 +73,11 @@ ot_keyfile_get_value_with_default (GKeyFile      *keyfile,
 {
   gboolean ret = FALSE;
   GError *temp_error = NULL;
-  gs_free char *ret_value = NULL;
+  g_autofree char *ret_value = NULL;
+
+  g_return_val_if_fail (keyfile != NULL, ret);
+  g_return_val_if_fail (section != NULL, ret);
+  g_return_val_if_fail (value != NULL, ret);
 
   ret_value = g_key_file_get_value (keyfile, section, value, &temp_error);
   if (temp_error)
@@ -91,3 +99,35 @@ ot_keyfile_get_value_with_default (GKeyFile      *keyfile,
  out:
   return ret;
 }
+
+gboolean
+ot_keyfile_copy_group (GKeyFile   *source_keyfile,
+                       GKeyFile   *target_keyfile,
+                       const char *group_name)
+{
+  g_auto(GStrv) keys = NULL;
+  gsize length, ii;
+  gboolean ret = FALSE;
+
+  g_return_val_if_fail (source_keyfile != NULL, ret);
+  g_return_val_if_fail (target_keyfile != NULL, ret);
+  g_return_val_if_fail (group_name != NULL, ret);
+
+  keys = g_key_file_get_keys (source_keyfile, group_name, &length, NULL);
+
+  if (keys == NULL)
+    goto out;
+
+  for (ii = 0; ii < length; ii++)
+    {
+      g_autofree char *value = NULL;
+
+      value = g_key_file_get_value (source_keyfile, group_name, keys[ii], NULL);
+      g_key_file_set_value (target_keyfile, group_name, keys[ii], value);
+    }
+
+  ret = TRUE;
+
+ out:
+  return ret;
+}
