@@ -26,6 +26,6 @@ G_BEGIN_DECLS
 
 void ot_log_structured_print_id_v (const char *message_id,
                                    const char *format,
-                                   ...);
+                                   ...) G_GNUC_PRINTF(2, 3);
 
 G_END_DECLS
