@@ -33,8 +33,7 @@
 GVariant *
 ot_gvariant_new_empty_string_dict (void)
 {
-  g_auto(GVariantBuilder) builder = {{0,}};
-
+  g_auto(GVariantBuilder) builder = OT_VARIANT_BUILDER_INITIALIZER;
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
   return g_variant_builder_end (&builder);
 }
