@@ -54,11 +54,10 @@ ot_admin_builtin_deploy (int argc, char **argv, GCancellable *cancellable, GErro
 {
   gboolean ret = FALSE;
   const char *refspec;
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
   glnx_unref_object OstreeSysroot *sysroot = NULL;
   GKeyFile *origin = NULL;
   glnx_unref_object OstreeRepo *repo = NULL;
-  g_autoptr(GPtrArray) new_deployments = NULL;
   glnx_unref_object OstreeDeployment *new_deployment = NULL;
   glnx_unref_object OstreeDeployment *merge_deployment = NULL;
   g_autofree char *revision = NULL;
@@ -174,7 +173,5 @@ ot_admin_builtin_deploy (int argc, char **argv, GCancellable *cancellable, GErro
  out:
   if (origin)
     g_key_file_unref (origin);
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
