@@ -43,7 +43,7 @@ gboolean
 ot_admin_builtin_switch (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
   gboolean ret = FALSE;
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
   glnx_unref_object OstreeSysroot *sysroot = NULL;
   const char *new_provided_refspec = NULL;
   glnx_unref_object OstreeRepo *repo = NULL;
@@ -53,11 +53,6 @@ ot_admin_builtin_switch (int argc, char **argv, GCancellable *cancellable, GErro
   g_autofree char *new_remote = NULL;
   g_autofree char *new_ref = NULL;
   g_autofree char *new_refspec = NULL;
-  g_autofree char *new_revision = NULL;
-  g_autoptr(GFile) deployment_path = NULL;
-  g_autoptr(GFile) deployment_origin_path = NULL;
-  glnx_unref_object OstreeDeployment *merge_deployment = NULL;
-  glnx_unref_object OstreeDeployment *new_deployment = NULL;
   glnx_unref_object OstreeSysrootUpgrader *upgrader = NULL;
   glnx_unref_object OstreeAsyncProgress *progress = NULL;
   gboolean changed;
@@ -167,7 +162,5 @@ ot_admin_builtin_switch (int argc, char **argv, GCancellable *cancellable, GErro
  out:
   if (new_origin)
     g_key_file_unref (new_origin);
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
