@@ -22,9 +22,9 @@
 
 #include "config.h"
 
+#include "ot-main.h"
 #include "ot-builtins.h"
 #include "ostree.h"
-#include "libgsystem.h"
 
 #include <string.h>
 
@@ -42,8 +42,8 @@ on_checksum_received (GObject    *obj,
                       GAsyncResult  *result,
                       gpointer       user_data)
 {
-  gs_free guchar *csum = NULL;
-  gs_free char *checksum = NULL;
+  g_autofree guchar *csum = NULL;
+  g_autofree char *checksum = NULL;
   AsyncChecksumData *data = user_data;
 
   if (ostree_checksum_file_async_finish ((GFile*)obj, result, &csum, data->error))
@@ -56,17 +56,16 @@ on_checksum_received (GObject    *obj,
 }
 
 gboolean
-ostree_builtin_checksum (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
+ostree_builtin_checksum (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
   gboolean ret = FALSE;
-  gs_unref_object GFile *f = NULL;
+  g_autoptr(GFile) f = NULL;
   AsyncChecksumData data = { 0, };
 
-  context = g_option_context_new ("FILENAME - Checksum a file or directory");
-  g_option_context_add_main_entries (context, options, NULL);
+  context = g_option_context_new ("PATH - Checksum a file or directory");
 
-  if (!g_option_context_parse (context, &argc, &argv, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NO_REPO, NULL, cancellable, error))
     goto out;
 
   if (argc > 1)
@@ -88,7 +87,5 @@ ostree_builtin_checksum (int argc, char **argv, OstreeRepo *repo, GCancellable *
  out:
   if (data.loop)
     g_main_loop_unref (data.loop);
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
