@@ -22,15 +22,22 @@
 
 #include "config.h"
 
+#include "ot-main.h"
 #include "ot-builtins.h"
 #include "ot-editor.h"
 #include "ostree.h"
 #include "otutil.h"
+#include "ot-tool-util.h"
+#include "parse-datetime.h"
 
 static char *opt_subject;
 static char *opt_body;
+static gboolean opt_editor;
+static char *opt_parent;
+static gboolean opt_orphan;
 static char *opt_branch;
 static char *opt_statoverride_file;
+static char *opt_skiplist_file;
 static char **opt_metadata_strings;
 static char **opt_detached_metadata_strings;
 static gboolean opt_link_checkout_speedup;
@@ -41,95 +48,136 @@ static char **opt_trees;
 static gint opt_owner_uid = -1;
 static gint opt_owner_gid = -1;
 static gboolean opt_table_output;
-#ifdef HAVE_GPGME
 static char **opt_key_ids;
 static char *opt_gpg_homedir;
-#endif
 static gboolean opt_generate_sizes;
+static gboolean opt_disable_fsync;
+static char *opt_timestamp;
+
+static gboolean
+parse_fsync_cb (const char  *option_name,
+                const char  *value,
+                gpointer     data,
+                GError     **error)
+{
+  gboolean val;
+
+  if (!ot_parse_boolean (value, &val, error))
+    return FALSE;
+    
+  opt_disable_fsync = !val;
+
+  return TRUE;
+}
 
 static GOptionEntry options[] = {
-  { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, "One line subject", "subject" },
-  { "body", 'm', 0, G_OPTION_ARG_STRING, &opt_body, "Full description", "body" },
-  { "branch", 'b', 0, G_OPTION_ARG_STRING, &opt_branch, "Branch", "branch" },
-  { "tree", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_trees, "Overlay the given argument as a tree", "NAME" },
-  { "add-metadata-string", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_metadata_strings, "Append given key and value (in string format) to metadata", "KEY=VALUE" },
-  { "add-detached-metadata-string", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_detached_metadata_strings, "Append given key and value (in string format) to detached metadata", "KEY=VALUE" },
+  { "parent", 0, 0, G_OPTION_ARG_STRING, &opt_parent, "Parent ref, or \"none\"", "REF" },
+  { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, "One line subject", "SUBJECT" },
+  { "body", 'm', 0, G_OPTION_ARG_STRING, &opt_body, "Full description", "BODY" },
+  { "editor", 'e', 0, G_OPTION_ARG_NONE, &opt_editor, "Use an editor to write the commit message", NULL },
+  { "branch", 'b', 0, G_OPTION_ARG_STRING, &opt_branch, "Branch", "BRANCH" },
+  { "orphan", 0, 0, G_OPTION_ARG_NONE, &opt_orphan, "Create a commit without writing a ref", NULL },
+  { "tree", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_trees, "Overlay the given argument as a tree", "dir=PATH or tar=TARFILE or ref=COMMIT" },
+  { "add-metadata-string", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_metadata_strings, "Add a key/value pair to metadata", "KEY=VALUE" },
+  { "add-detached-metadata-string", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_detached_metadata_strings, "Add a key/value pair to detached metadata", "KEY=VALUE" },
   { "owner-uid", 0, 0, G_OPTION_ARG_INT, &opt_owner_uid, "Set file ownership user id", "UID" },
   { "owner-gid", 0, 0, G_OPTION_ARG_INT, &opt_owner_gid, "Set file ownership group id", "GID" },
   { "no-xattrs", 0, 0, G_OPTION_ARG_NONE, &opt_no_xattrs, "Do not import extended attributes", NULL },
   { "link-checkout-speedup", 0, 0, G_OPTION_ARG_NONE, &opt_link_checkout_speedup, "Optimize for commits of trees composed of hardlinks into the repository", NULL },
   { "tar-autocreate-parents", 0, 0, G_OPTION_ARG_NONE, &opt_tar_autocreate_parents, "When loading tar archives, automatically create parent directories as needed", NULL },
   { "skip-if-unchanged", 0, 0, G_OPTION_ARG_NONE, &opt_skip_if_unchanged, "If the contents are unchanged from previous commit, do nothing", NULL },
-  { "statoverride", 0, 0, G_OPTION_ARG_FILENAME, &opt_statoverride_file, "File containing list of modifications to make to permissions", "path" },
+  { "statoverride", 0, 0, G_OPTION_ARG_FILENAME, &opt_statoverride_file, "File containing list of modifications to make to permissions", "PATH" },
+  { "skip-list", 0, 0, G_OPTION_ARG_FILENAME, &opt_skiplist_file, "File containing list of files to skip", "PATH" },
   { "table-output", 0, 0, G_OPTION_ARG_NONE, &opt_table_output, "Output more information in a KEY: VALUE format", NULL },
-#ifdef HAVE_GPGME
-  { "gpg-sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_key_ids, "GPG Key ID to sign the commit with", "key-id"},
-  { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, "GPG Homedir to use when looking for keyrings", "homedir"},
-#endif
+  { "gpg-sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_key_ids, "GPG Key ID to sign the commit with", "KEY-ID"},
+  { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, "GPG Homedir to use when looking for keyrings", "HOMEDIR"},
   { "generate-sizes", 0, 0, G_OPTION_ARG_NONE, &opt_generate_sizes, "Generate size information along with commit metadata", NULL },
+  { "disable-fsync", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &opt_disable_fsync, "Do not invoke fsync()", NULL },
+  { "fsync", 0, 0, G_OPTION_ARG_CALLBACK, parse_fsync_cb, "Specify how to invoke fsync()", "POLICY" },
+  { "timestamp", 0, 0, G_OPTION_ARG_STRING, &opt_timestamp, "Override the timestamp of the commit", "TIMESTAMP" },
   { NULL }
 };
 
 static gboolean
-parse_statoverride_file (GHashTable   **out_mode_add,
-                         GCancellable  *cancellable,
-                         GError        **error)
+parse_file_by_line (const char    *path,
+                    gboolean     (*cb)(const char*, void*, GError**),
+                    void          *cbdata,
+                    GCancellable  *cancellable,
+                    GError       **error)
 {
   gboolean ret = FALSE;
-  gsize len;
-  char **iter = NULL; /* nofree */
-  gs_unref_hashtable GHashTable *ret_hash = NULL;
-  gs_unref_object GFile *path = NULL;
-  gs_free char *contents = NULL;
+  g_autofree char *contents = NULL;
+  g_autoptr(GFile) file = NULL;
   char **lines = NULL;
 
-  path = g_file_new_for_path (opt_statoverride_file);
-
-  if (!g_file_load_contents (path, cancellable, &contents, &len, NULL,
-                             error))
+  file = g_file_new_for_path (path);
+  if (!g_file_load_contents (file, cancellable, &contents, NULL, NULL, error))
     goto out;
-  
-  ret_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
   lines = g_strsplit (contents, "\n", -1);
-
-  for (iter = lines; iter && *iter; iter++)
+  for (char **iter = lines; iter && *iter; iter++)
     {
-      const char *line = *iter;
+      /* skip empty lines at least */
+      if (**iter == '\0')
+        continue;
 
-      if (*line == '+')
-        {
-          const char *spc;
-          guint mode_add;
-
-          spc = strchr (line + 1, ' ');
-          if (!spc)
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "Malformed statoverride file");
-              goto out;
-            }
-          
-          mode_add = (guint32)(gint32)g_ascii_strtod (line + 1, NULL);
-          g_hash_table_insert (ret_hash,
-                               g_strdup (spc + 1),
-                               GUINT_TO_POINTER((gint32)mode_add));
-        }
+      if (!cb (*iter, cbdata, error))
+        goto out;
     }
 
   ret = TRUE;
-  ot_transfer_out_value (out_mode_add, &ret_hash);
- out:
+out:
   g_strfreev (lines);
   return ret;
 }
 
+static gboolean
+handle_statoverride_line (const char  *line,
+                          void        *data,
+                          GError     **error)
+{
+  GHashTable *files = data;
+  const char *spc;
+  guint mode_add;
+
+  spc = strchr (line, ' ');
+  if (spc == NULL)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Malformed statoverride file (no space found)");
+      return FALSE;
+    }
+
+  mode_add = (guint32)(gint32)g_ascii_strtod (line, NULL);
+  g_hash_table_insert (files, g_strdup (spc + 1),
+                       GUINT_TO_POINTER((gint32)mode_add));
+  return TRUE;
+}
+
+static gboolean
+handle_skiplist_line (const char  *line,
+                      void        *data,
+                      GError     **error)
+{
+  GHashTable *files = data;
+  g_hash_table_add (files, g_strdup (line));
+  return TRUE;
+}
+
+struct CommitFilterData {
+  GHashTable *mode_adds;
+  GHashTable *skip_list;
+};
+
 static OstreeRepoCommitFilterResult
 commit_filter (OstreeRepo         *self,
                const char         *path,
                GFileInfo          *file_info,
                gpointer            user_data)
 {
-  GHashTable *mode_adds = user_data;
+  struct CommitFilterData *data = user_data;
+  GHashTable *mode_adds = data->mode_adds;
+  GHashTable *skip_list = data->skip_list;
   gpointer value;
 
   if (opt_owner_uid >= 0)
@@ -145,7 +193,13 @@ commit_filter (OstreeRepo         *self,
                                         current_mode | mode_add);
       g_hash_table_remove (mode_adds, path);
     }
-  
+
+  if (skip_list && g_hash_table_contains (skip_list, path))
+    {
+      g_hash_table_remove (skip_list, path);
+      return OSTREE_REPO_COMMIT_FILTER_SKIP;
+    }
+
   return OSTREE_REPO_COMMIT_FILTER_ALLOW;
 }
 
@@ -157,22 +211,25 @@ commit_editor (OstreeRepo     *repo,
                GCancellable   *cancellable,
                GError        **error)
 {
-  gs_free char *input = NULL;
-  gs_free char *output = NULL;
+  g_autofree char *input = NULL;
+  g_autofree char *output = NULL;
   gboolean ret = FALSE;
   GString *bodybuf = NULL;
   char **lines = NULL;
   int i;
 
-  *subject = NULL;
-  *body = NULL;
-
   input = g_strdup_printf ("\n"
       "# Please enter the commit message for your changes. The first line will\n"
       "# become the subject, and the remainder the body. Lines starting\n"
-      "# with '#' will be ignored, and an empty message aborts the commit.\n"
-      "#\n"
-      "# Branch: %s\n", branch);
+      "# with '#' will be ignored, and an empty message aborts the commit."
+      "%s%s%s%s%s%s\n"
+              , branch ? "\n#\n# Branch: " : "", branch ? branch : ""
+              , *subject ? "\n" : "", *subject ? *subject : ""
+              , *body ? "\n" : "", *body ? *body : ""
+              );
+
+  *subject = NULL;
+  *body = NULL;
 
   output = ot_editor_prompt (repo, input, cancellable, error);
   if (output == NULL)
@@ -239,7 +296,7 @@ parse_keyvalue_strings (char             **strings,
 {
   gboolean ret = FALSE;
   char **iter;
-  gs_unref_variant_builder GVariantBuilder *builder = NULL;
+  g_autoptr(GVariantBuilder) builder = NULL;
 
   builder = g_variant_builder_new (G_VARIANT_TYPE ("a{sv}"));
 
@@ -247,7 +304,7 @@ parse_keyvalue_strings (char             **strings,
     {
       const char *s;
       const char *eq;
-      gs_free char *key = NULL;
+      g_autofree char *key = NULL;
 
       s = *iter;
 
@@ -272,33 +329,48 @@ parse_keyvalue_strings (char             **strings,
 }
 
 gboolean
-ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
+ostree_builtin_commit (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
+  glnx_unref_object OstreeRepo *repo = NULL;
   gboolean ret = FALSE;
   gboolean skip_commit = FALSE;
-  gs_unref_object GFile *arg = NULL;
-  gs_free char *parent = NULL;
-  gs_free char *commit_checksum = NULL;
-  gs_unref_object GFile *root = NULL;
-  gs_unref_variant GVariant *metadata = NULL;
-  gs_unref_variant GVariant *detached_metadata = NULL;
-  gs_unref_object OstreeMutableTree *mtree = NULL;
-  gs_free char *tree_type = NULL;
-  gs_unref_hashtable GHashTable *mode_adds = NULL;
+  g_autoptr(GFile) object_to_commit = NULL;
+  g_autofree char *parent = NULL;
+  g_autofree char *commit_checksum = NULL;
+  g_autoptr(GFile) root = NULL;
+  g_autoptr(GVariant) metadata = NULL;
+  g_autoptr(GVariant) detached_metadata = NULL;
+  glnx_unref_object OstreeMutableTree *mtree = NULL;
+  g_autofree char *tree_type = NULL;
+  g_autoptr(GHashTable) mode_adds = NULL;
+  g_autoptr(GHashTable) skip_list = NULL;
   OstreeRepoCommitModifierFlags flags = 0;
   OstreeRepoCommitModifier *modifier = NULL;
   OstreeRepoTransactionStats stats;
+  struct CommitFilterData filter_data = { 0, };
 
-  context = g_option_context_new ("[ARG] - Commit a new revision");
-  g_option_context_add_main_entries (context, options, NULL);
+  context = g_option_context_new ("[PATH] - Commit a new revision");
 
-  if (!g_option_context_parse (context, &argc, &argv, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NONE, &repo, cancellable, error))
+    goto out;
+
+  if (!ostree_ensure_repo_writable (repo, error))
     goto out;
 
   if (opt_statoverride_file)
     {
-      if (!parse_statoverride_file (&mode_adds, cancellable, error))
+      mode_adds = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+      if (!parse_file_by_line (opt_statoverride_file, handle_statoverride_line,
+                               mode_adds, cancellable, error))
+        goto out;
+    }
+
+  if (opt_skiplist_file)
+    {
+      skip_list = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+      if (!parse_file_by_line (opt_skiplist_file, handle_skiplist_line,
+                               skip_list, cancellable, error))
         goto out;
     }
 
@@ -315,10 +387,10 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
         goto out;
     }
       
-  if (!opt_branch)
+  if (!(opt_branch || opt_orphan))
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "A branch must be specified with --branch");
+                           "A branch must be specified with --branch, or use --orphan");
       goto out;
     }
 
@@ -326,32 +398,53 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
     flags |= OSTREE_REPO_COMMIT_MODIFIER_FLAGS_SKIP_XATTRS;
   if (opt_generate_sizes)
     flags |= OSTREE_REPO_COMMIT_MODIFIER_FLAGS_GENERATE_SIZES;
+  if (opt_disable_fsync)
+    ostree_repo_set_disable_fsync (repo, TRUE);
 
   if (flags != 0
       || opt_owner_uid >= 0
       || opt_owner_gid >= 0
       || opt_statoverride_file != NULL
+      || opt_skiplist_file != NULL
       || opt_no_xattrs)
     {
-      modifier = ostree_repo_commit_modifier_new (flags, commit_filter, mode_adds, NULL);
+      filter_data.mode_adds = mode_adds;
+      filter_data.skip_list = skip_list;
+      modifier = ostree_repo_commit_modifier_new (flags, commit_filter,
+                                                  &filter_data, NULL);
     }
 
-  if (!ostree_repo_resolve_rev (repo, opt_branch, TRUE, &parent, error))
-    goto out;
+  if (opt_parent)
+    {
+      if (g_str_equal (opt_parent, "none"))
+        parent = NULL;
+      else
+        {
+          if (!ostree_validate_checksum_string (opt_parent, error))
+            goto out;
+          parent = g_strdup (opt_parent);
+        }
+    }
+  else if (!opt_orphan)
+    {
+      if (!ostree_repo_resolve_rev (repo, opt_branch, TRUE, &parent, error))
+        {
+          if (g_error_matches (*error, G_IO_ERROR, G_IO_ERROR_IS_DIRECTORY))
+            {
+              /* A folder exists with the specified ref name,
+                 * which is handled by _ostree_repo_write_ref */
+              g_clear_error (error);
+            }
+          else goto out;
+        }
+    }
 
-  if (!opt_subject && !opt_body)
+  if (opt_editor)
     {
       if (!commit_editor (repo, opt_branch, &opt_subject, &opt_body, cancellable, error))
         goto out;
     }
 
-  if (!opt_subject)
-    {
-      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "A subject must be specified with --subject");
-      goto out;
-    }
-
   if (!ostree_repo_prepare_transaction (repo, NULL, cancellable, error))
     goto out;
 
@@ -362,12 +455,8 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
 
   if (argc <= 1 && (opt_trees == NULL || opt_trees[0] == NULL))
     {
-      char *current_dir = g_get_current_dir ();
-      arg = g_file_new_for_path (current_dir);
-      g_free (current_dir);
-
-      if (!ostree_repo_write_directory_to_mtree (repo, arg, mtree, modifier,
-                                                 cancellable, error))
+      if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, ".", mtree, modifier,
+                                           cancellable, error))
         goto out;
     }
   else if (opt_trees != NULL)
@@ -391,28 +480,27 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
           tree_type = g_strndup (tree, eq - tree);
           tree = eq + 1;
 
-          g_clear_object (&arg);
+          g_clear_object (&object_to_commit);
           if (strcmp (tree_type, "dir") == 0)
             {
-              arg = g_file_new_for_path (tree);
-              if (!ostree_repo_write_directory_to_mtree (repo, arg, mtree, modifier,
-                                                         cancellable, error))
+              if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, tree, mtree, modifier,
+                                                   cancellable, error))
                 goto out;
             }
           else if (strcmp (tree_type, "tar") == 0)
             {
-              arg = g_file_new_for_path (tree);
-              if (!ostree_repo_write_archive_to_mtree (repo, arg, mtree, modifier,
+              object_to_commit = g_file_new_for_path (tree);
+              if (!ostree_repo_write_archive_to_mtree (repo, object_to_commit, mtree, modifier,
                                                        opt_tar_autocreate_parents,
                                                        cancellable, error))
                 goto out;
             }
           else if (strcmp (tree_type, "ref") == 0)
             {
-              if (!ostree_repo_read_commit (repo, tree, &arg, NULL, cancellable, error))
+              if (!ostree_repo_read_commit (repo, tree, &object_to_commit, NULL, cancellable, error))
                 goto out;
 
-              if (!ostree_repo_write_directory_to_mtree (repo, arg, mtree, modifier,
+              if (!ostree_repo_write_directory_to_mtree (repo, object_to_commit, mtree, modifier,
                                                          cancellable, error))
                 goto out;
             }
@@ -427,9 +515,8 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
   else
     {
       g_assert (argc > 1);
-      arg = g_file_new_for_path (argv[1]);
-      if (!ostree_repo_write_directory_to_mtree (repo, arg, mtree, modifier,
-                                                 cancellable, error))
+      if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, argv[1], mtree, modifier,
+                                           cancellable, error))
         goto out;
     }
 
@@ -449,12 +536,28 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
       goto out;
     }
 
+  if (skip_list && g_hash_table_size (skip_list) > 0)
+    {
+      GHashTableIter hash_iter;
+      gpointer key;
+
+      g_hash_table_iter_init (&hash_iter, skip_list);
+
+      while (g_hash_table_iter_next (&hash_iter, &key, NULL))
+        {
+          g_printerr ("Unmatched skip-list path: %s\n", (char*)key);
+        }
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Unmatched skip-list paths");
+      goto out;
+    }
+
   if (!ostree_repo_write_mtree (repo, mtree, &root, cancellable, error))
     goto out;
 
   if (opt_skip_if_unchanged && parent)
     {
-      gs_unref_object GFile *parent_root;
+      g_autoptr(GFile) parent_root;
 
       if (!ostree_repo_read_commit (repo, parent, &parent_root, NULL, cancellable, error))
         goto out;
@@ -465,10 +568,36 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
 
   if (!skip_commit)
     {
-      if (!ostree_repo_write_commit (repo, parent, opt_subject, opt_body, metadata,
-                                     OSTREE_REPO_FILE (root),
-                                     &commit_checksum, cancellable, error))
-        goto out;
+      gboolean update_summary;
+      guint64 timestamp;
+      if (!opt_timestamp)
+        {
+          GDateTime *now = g_date_time_new_now_utc ();
+          timestamp = g_date_time_to_unix (now);
+          g_date_time_unref (now);
+
+          if (!ostree_repo_write_commit (repo, parent, opt_subject, opt_body, metadata,
+                                         OSTREE_REPO_FILE (root),
+                                         &commit_checksum, cancellable, error))
+            goto out;
+        }
+      else
+        {
+          struct timespec ts;
+          if (!parse_datetime (&ts, opt_timestamp, NULL))
+            {
+              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                           "Could not parse '%s'", opt_timestamp);
+              goto out;
+            }
+          timestamp = ts.tv_sec;
+
+          if (!ostree_repo_write_commit_with_time (repo, parent, opt_subject, opt_body, metadata,
+                                                   OSTREE_REPO_FILE (root),
+                                                   timestamp,
+                                                   &commit_checksum, cancellable, error))
+            goto out;
+        }
 
       if (detached_metadata)
         {
@@ -478,7 +607,6 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
             goto out;
         }
 
-#ifdef HAVE_GPGME
       if (opt_key_ids)
         {
           char **iter;
@@ -496,12 +624,30 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
                 goto out;
             }
         }
-#endif
 
-      ostree_repo_transaction_set_ref (repo, NULL, opt_branch, commit_checksum);
+      if (opt_branch)
+        ostree_repo_transaction_set_ref (repo, NULL, opt_branch, commit_checksum);
+      else
+        g_assert (opt_orphan);
 
       if (!ostree_repo_commit_transaction (repo, &stats, cancellable, error))
         goto out;
+
+      /* The default for this option is FALSE, even for archive-z2 repos,
+       * because ostree supports multiple processes committing to the same
+       * repo (but different refs) concurrently, and in fact gnome-continuous
+       * actually does this.  In that context it's best to update the summary
+       * explicitly instead of automatically here. */
+      if (!ot_keyfile_get_boolean_with_default (ostree_repo_get_config (repo), "core",
+                                                "commit-update-summary", FALSE,
+                                                &update_summary, error))
+        goto out;
+
+      if (update_summary && !ostree_repo_regenerate_summary (repo,
+                                                             NULL,
+                                                             cancellable,
+                                                             error))
+        goto out;
     }
   else
     {
@@ -524,9 +670,8 @@ ostree_builtin_commit (int argc, char **argv, OstreeRepo *repo, GCancellable *ca
 
   ret = TRUE;
  out:
-  ostree_repo_abort_transaction (repo, cancellable, NULL);
-  if (context)
-    g_option_context_free (context);
+  if (repo)
+    ostree_repo_abort_transaction (repo, cancellable, NULL);
   if (modifier)
     ostree_repo_commit_modifier_unref (modifier);
   return ret;
