@@ -22,16 +22,19 @@
 
 #include "config.h"
 
+#include "ot-main.h"
 #include "ot-builtins.h"
 #include "ostree.h"
 #include "otutil.h"
 
 static gboolean opt_stats;
 static gboolean opt_fs_diff;
+static gboolean opt_no_xattrs;
 
 static GOptionEntry options[] = {
   { "stats", 0, 0, G_OPTION_ARG_NONE, &opt_stats, "Print various statistics", NULL },
   { "fs-diff", 0, 0, G_OPTION_ARG_NONE, &opt_fs_diff, "Print filesystem diff", NULL },
+  { "no-xattrs", 0, 0, G_OPTION_ARG_NONE, &opt_no_xattrs, "Skip output of extended attributes", NULL },
   { NULL }
 };
 
@@ -43,7 +46,7 @@ parse_file_or_commit (OstreeRepo  *repo,
                       GError     **error)
 {
   gboolean ret = FALSE;
-  gs_free GFile *ret_file = NULL;
+  g_autoptr(GFile) ret_file = NULL;
 
   if (g_str_has_prefix (arg, "/")
       || g_str_has_prefix (arg, "./")
@@ -115,23 +118,23 @@ object_set_total_size (OstreeRepo    *repo,
 }
 
 gboolean
-ostree_builtin_diff (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
+ostree_builtin_diff (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
   gboolean ret = FALSE;
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
+  glnx_unref_object OstreeRepo *repo = NULL;
   const char *src;
   const char *target;
-  gs_free char *src_prev = NULL;
-  gs_unref_object GFile *srcf = NULL;
-  gs_unref_object GFile *targetf = NULL;
-  gs_unref_ptrarray GPtrArray *modified = NULL;
-  gs_unref_ptrarray GPtrArray *removed = NULL;
-  gs_unref_ptrarray GPtrArray *added = NULL;
+  g_autofree char *src_prev = NULL;
+  g_autoptr(GFile) srcf = NULL;
+  g_autoptr(GFile) targetf = NULL;
+  g_autoptr(GPtrArray) modified = NULL;
+  g_autoptr(GPtrArray) removed = NULL;
+  g_autoptr(GPtrArray) added = NULL;
 
   context = g_option_context_new ("REV TARGETDIR - Compare directory TARGETDIR against revision REV");
-  g_option_context_add_main_entries (context, options, NULL);
 
-  if (!g_option_context_parse (context, &argc, &argv, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NONE, &repo, cancellable, error))
     goto out;
 
   if (argc < 2)
@@ -161,6 +164,11 @@ ostree_builtin_diff (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
 
   if (opt_fs_diff)
     {
+      OstreeDiffFlags diff_flags = OSTREE_DIFF_FLAGS_NONE; 
+
+      if (opt_no_xattrs)
+        diff_flags |= OSTREE_DIFF_FLAGS_IGNORE_XATTRS;
+      
       if (!parse_file_or_commit (repo, src, &srcf, cancellable, error))
         goto out;
       if (!parse_file_or_commit (repo, target, &targetf, cancellable, error))
@@ -170,7 +178,7 @@ ostree_builtin_diff (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
       removed = g_ptr_array_new_with_free_func ((GDestroyNotify)g_object_unref);
       added = g_ptr_array_new_with_free_func ((GDestroyNotify)g_object_unref);
       
-      if (!ostree_diff_dirs (OSTREE_DIFF_FLAGS_NONE, srcf, targetf, modified, removed, added, cancellable, error))
+      if (!ostree_diff_dirs (diff_flags, srcf, targetf, modified, removed, added, cancellable, error))
         goto out;
 
       ostree_diff_print (srcf, targetf, modified, removed, added);
@@ -178,12 +186,12 @@ ostree_builtin_diff (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
 
   if (opt_stats)
     {
-      gs_unref_hashtable GHashTable *reachable_a = NULL;
-      gs_unref_hashtable GHashTable *reachable_b = NULL;
-      gs_unref_hashtable GHashTable *reachable_intersection = NULL;
-      gs_free char *rev_a = NULL;
-      gs_free char *rev_b = NULL;
-      gs_free char *size = NULL;
+      g_autoptr(GHashTable) reachable_a = NULL;
+      g_autoptr(GHashTable) reachable_b = NULL;
+      g_autoptr(GHashTable) reachable_intersection = NULL;
+      g_autofree char *rev_a = NULL;
+      g_autofree char *rev_b = NULL;
+      g_autofree char *size = NULL;
       guint a_size;
       guint b_size;
       guint64 total_common;
@@ -193,9 +201,9 @@ ostree_builtin_diff (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
       if (!ostree_repo_resolve_rev (repo, target, FALSE, &rev_b, error))
         goto out;
 
-      if (!ostree_repo_traverse_commit (repo, rev_a, -1, &reachable_a, cancellable, error))
+      if (!ostree_repo_traverse_commit (repo, rev_a, 0, &reachable_a, cancellable, error))
         goto out;
-      if (!ostree_repo_traverse_commit (repo, rev_b, -1, &reachable_b, cancellable, error))
+      if (!ostree_repo_traverse_commit (repo, rev_b, 0, &reachable_b, cancellable, error))
         goto out;
 
       a_size = g_hash_table_size (reachable_a);
@@ -216,7 +224,5 @@ ostree_builtin_diff (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
 
   ret = TRUE;
  out:
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
