@@ -22,14 +22,18 @@
 
 #include "config.h"
 
+#include "ot-main.h"
 #include "ot-builtins.h"
 #include "ostree.h"
+#include "ostree-cmdprivate.h"
 #include "otutil.h"
 
 static gboolean opt_quiet;
 static gboolean opt_delete;
+static gboolean opt_add_tombstones;
 
 static GOptionEntry options[] = {
+  { "add-tombstones", 0, 0, G_OPTION_ARG_NONE, &opt_add_tombstones, "Add tombstones for missing commits", NULL },
   { "quiet", 'q', 0, G_OPTION_ARG_NONE, &opt_quiet, "Only print error messages", NULL },
   { "delete", 0, 0, G_OPTION_ARG_NONE, &opt_delete, "Remove corrupted objects", NULL },
   { NULL }
@@ -45,10 +49,10 @@ load_and_fsck_one_object (OstreeRepo            *repo,
 {
   gboolean ret = FALSE;
   gboolean missing = FALSE;
-  gs_unref_variant GVariant *metadata = NULL;
-  gs_unref_object GInputStream *input = NULL;
-  gs_unref_object GFileInfo *file_info = NULL;
-  gs_unref_variant GVariant *xattrs = NULL;
+  g_autoptr(GVariant) metadata = NULL;
+  g_autoptr(GInputStream) input = NULL;
+  g_autoptr(GFileInfo) file_info = NULL;
+  g_autoptr(GVariant) xattrs = NULL;
   GError *temp_error = NULL;
 
   if (OSTREE_OBJECT_TYPE_IS_META (objtype))
@@ -118,6 +122,7 @@ load_and_fsck_one_object (OstreeRepo            *repo,
             }
           else
             {
+              *error = temp_error;
               g_prefix_error (error, "Loading file object %s: ", checksum);
               goto out;
             }
@@ -139,8 +144,8 @@ load_and_fsck_one_object (OstreeRepo            *repo,
     }
   else
     {
-      gs_free guchar *computed_csum = NULL;
-      gs_free char *tmp_checksum = NULL;
+      g_autofree guchar *computed_csum = NULL;
+      g_autofree char *tmp_checksum = NULL;
 
       if (!ostree_checksum_file_from_input (file_info, xattrs, input,
                                             objtype, &computed_csum,
@@ -150,7 +155,7 @@ load_and_fsck_one_object (OstreeRepo            *repo,
       tmp_checksum = ostree_checksum_from_bytes (computed_csum);
       if (strcmp (checksum, tmp_checksum) != 0)
         {
-          gs_free char *msg = g_strdup_printf ("corrupted object %s.%s; actual checksum: %s",
+          g_autofree char *msg = g_strdup_printf ("corrupted object %s.%s; actual checksum: %s",
                                                checksum, ostree_object_type_to_string (objtype),
                                                tmp_checksum);
           if (opt_delete)
@@ -182,7 +187,7 @@ fsck_reachable_objects_from_commits (OstreeRepo            *repo,
   gboolean ret = FALSE;
   GHashTableIter hash_iter;
   gpointer key, value;
-  gs_unref_hashtable GHashTable *reachable_objects = NULL;
+  g_autoptr(GHashTable) reachable_objects = NULL;
   guint i;
   guint mod;
   guint count;
@@ -222,7 +227,7 @@ fsck_reachable_objects_from_commits (OstreeRepo            *repo,
         goto out;
 
       if (mod == 0 || (i % mod == 0))
-        g_print ("%u/%u objects\n", i, count);
+        g_print ("%u/%u objects\n", i + 1, count);
       i++;
     }
 
@@ -232,20 +237,21 @@ fsck_reachable_objects_from_commits (OstreeRepo            *repo,
 }
 
 gboolean
-ostree_builtin_fsck (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
+ostree_builtin_fsck (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
   gboolean ret = FALSE;
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
+  glnx_unref_object OstreeRepo *repo = NULL;
   GHashTableIter hash_iter;
   gpointer key, value;
   gboolean found_corruption = FALSE;
-  gs_unref_hashtable GHashTable *objects = NULL;
-  gs_unref_hashtable GHashTable *commits = NULL;
-
+  guint n_partial = 0;
+  g_autoptr(GHashTable) objects = NULL;
+  g_autoptr(GHashTable) commits = NULL;
+  g_autoptr(GPtrArray) tombstones = NULL;
   context = g_option_context_new ("- Check the repository for consistency");
-  g_option_context_add_main_entries (context, options, NULL);
 
-  if (!g_option_context_parse (context, &argc, &argv, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NONE, &repo, cancellable, error))
     goto out;
 
   if (!opt_quiet)
@@ -260,16 +266,53 @@ ostree_builtin_fsck (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
   
   g_hash_table_iter_init (&hash_iter, objects);
 
+  if (opt_add_tombstones)
+    tombstones = g_ptr_array_new_with_free_func (g_free);
+
   while (g_hash_table_iter_next (&hash_iter, &key, &value))
     {
       GVariant *serialized_key = key;
       const char *checksum;
       OstreeObjectType objtype;
+      OstreeRepoCommitState commitstate = 0;
+      g_autoptr(GVariant) commit = NULL;
 
       ostree_object_name_deserialize (serialized_key, &checksum, &objtype);
 
       if (objtype == OSTREE_OBJECT_TYPE_COMMIT)
-        g_hash_table_insert (commits, g_variant_ref (serialized_key), serialized_key);
+        {
+          if (!ostree_repo_load_commit (repo, checksum, &commit, &commitstate, error))
+            goto out;
+
+          if (opt_add_tombstones)
+            {
+              GError *local_error = NULL;
+              g_autofree char *parent = ostree_commit_get_parent (commit);
+              if (parent)
+                {
+                  g_autoptr(GVariant) parent_commit = NULL;
+                  if (!ostree_repo_load_variant (repo, OSTREE_OBJECT_TYPE_COMMIT, parent,
+                                                 &parent_commit, &local_error))
+                    {
+                      if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+                        {
+                          g_ptr_array_add (tombstones, g_strdup (checksum));
+                          g_clear_error (&local_error);
+                        }
+                      else
+                        {
+                          g_propagate_error (error, local_error);
+                          goto out;
+                        }
+                    }
+                }
+            }
+
+          if (commitstate & OSTREE_REPO_COMMIT_STATE_PARTIAL)
+            n_partial++;
+          else
+            g_hash_table_add (commits, g_variant_ref (serialized_key));
+        }
     }
 
   g_clear_pointer (&objects, (GDestroyNotify) g_hash_table_unref);
@@ -282,6 +325,27 @@ ostree_builtin_fsck (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
                                             cancellable, error))
     goto out;
 
+  if (opt_add_tombstones)
+    {
+      guint i;
+      if (tombstones->len)
+        {
+          if (!ot_enable_tombstone_commits (repo, error))
+            goto out;
+        }
+      for (i = 0; i < tombstones->len; i++)
+        {
+          const char *checksum = tombstones->pdata[i];
+          g_print ("Adding tombstone for commit %s\n", checksum);
+          if (!ostree_repo_delete_object (repo, OSTREE_OBJECT_TYPE_COMMIT, checksum, cancellable, error))
+            goto out;
+        }
+    }
+  else if (n_partial > 0)
+    {
+      g_print ("%u partial commits not verified\n", n_partial);
+    }
+
   if (found_corruption)
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
@@ -291,7 +355,5 @@ ostree_builtin_fsck (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
 
   ret = TRUE;
  out:
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
