@@ -22,6 +22,7 @@
 
 #include "config.h"
 
+#include "ot-main.h"
 #include "ot-builtins.h"
 #include "ostree.h"
 #include "ostree-repo-file.h"
@@ -153,9 +154,9 @@ print_directory_recurse (GFile    *f,
                          GError  **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFileEnumerator *dir_enum = NULL;
-  gs_unref_object GFile *child = NULL;
-  gs_unref_object GFileInfo *child_info = NULL;
+  g_autoptr(GFileEnumerator) dir_enum = NULL;
+  g_autoptr(GFile) child = NULL;
+  g_autoptr(GFileInfo) child_info = NULL;
   GError *temp_error = NULL;
 
   if (depth > 0)
@@ -206,8 +207,8 @@ print_one_argument (OstreeRepo   *repo,
                     GError      **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFile *f = NULL;
-  gs_unref_object GFileInfo *file_info = NULL;
+  g_autoptr(GFile) f = NULL;
+  g_autoptr(GFileInfo) file_info = NULL;
 
   f = g_file_resolve_relative_path (root, arg);
   
@@ -239,18 +240,18 @@ print_one_argument (OstreeRepo   *repo,
 }
 
 gboolean
-ostree_builtin_ls (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
+ostree_builtin_ls (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
+  glnx_unref_object OstreeRepo *repo = NULL;
   gboolean ret = FALSE;
   const char *rev;
   int i;
-  gs_unref_object GFile *root = NULL;
+  g_autoptr(GFile) root = NULL;
 
-  context = g_option_context_new ("COMMIT PATH [PATH...] - List file paths");
-  g_option_context_add_main_entries (context, options, NULL);
+  context = g_option_context_new ("COMMIT [PATH...] - List file paths");
 
-  if (!g_option_context_parse (context, &argc, &argv, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NONE, &repo, cancellable, error))
     goto out;
 
   if (argc <= 1)
@@ -279,7 +280,5 @@ ostree_builtin_ls (int argc, char **argv, OstreeRepo *repo, GCancellable *cancel
   
   ret = TRUE;
  out:
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
