@@ -34,7 +34,8 @@ static gboolean opt_dry_run;
 static gboolean opt_disable_static_deltas;
 static gboolean opt_require_static_deltas;
 static gboolean opt_untrusted;
-static char* opt_subpath;
+static char** opt_subpaths;
+static char** opt_http_headers;
 static char* opt_cache_dir;
 static int opt_depth = 0;
 static char* opt_url;
@@ -46,11 +47,12 @@ static GOptionEntry options[] = {
    { "disable-static-deltas", 0, 0, G_OPTION_ARG_NONE, &opt_disable_static_deltas, "Do not use static deltas", NULL },
    { "require-static-deltas", 0, 0, G_OPTION_ARG_NONE, &opt_require_static_deltas, "Require static deltas", NULL },
    { "mirror", 0, 0, G_OPTION_ARG_NONE, &opt_mirror, "Write refs suitable for a mirror", NULL },
-   { "subpath", 0, 0, G_OPTION_ARG_STRING, &opt_subpath, "Only pull the provided subpath", NULL },
+   { "subpath", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_subpaths, "Only pull the provided subpath(s)", NULL },
    { "untrusted", 0, 0, G_OPTION_ARG_NONE, &opt_untrusted, "Do not trust (local) sources", NULL },
    { "dry-run", 0, 0, G_OPTION_ARG_NONE, &opt_dry_run, "Only print information on what will be downloaded (requires static deltas)", NULL },
    { "depth", 0, 0, G_OPTION_ARG_INT, &opt_depth, "Traverse DEPTH parents (-1=infinite) (default: 0)", "DEPTH" },
    { "url", 0, 0, G_OPTION_ARG_STRING, &opt_url, "Pull objects from this URL instead of the one from the remote config", NULL },
+   { "http-header", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_http_headers, "Add NAME=VALUE as HTTP header to all requests", "NAME=VALUE" },
    { NULL }
  };
 
@@ -107,7 +109,7 @@ dry_run_console_progress_changed (OstreeAsyncProgress *progress,
 gboolean
 ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
   glnx_unref_object OstreeRepo *repo = NULL;
   gboolean ret = FALSE;
   g_autofree char *remote = NULL;
@@ -198,7 +200,7 @@ ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **
   else
     {
       char *ref_to_fetch;
-      refs_to_fetch = g_ptr_array_new ();
+      refs_to_fetch = g_ptr_array_new_with_free_func (g_free);
       if (!ostree_parse_refspec (argv[1], &remote, &ref_to_fetch, error))
         goto out;
       /* Transfer ownership */
@@ -208,6 +210,7 @@ ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **
 
   {
     GVariantBuilder builder;
+    g_autoptr(GVariant) options = NULL;
     g_auto(GLnxConsoleRef) console = { 0, };
     g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
 
@@ -216,9 +219,17 @@ ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **
     if (opt_url)
       g_variant_builder_add (&builder, "{s@v}", "override-url",
                              g_variant_new_variant (g_variant_new_string (opt_url)));
-    if (opt_subpath)
-      g_variant_builder_add (&builder, "{s@v}", "subdir",
-                             g_variant_new_variant (g_variant_new_string (opt_subpath)));
+    if (opt_subpaths && opt_subpaths[0] != NULL)
+      {
+        /* Special case the one-element case so that we excercise this
+           old single-argument version in the tests */
+        if (opt_subpaths[1] == NULL)
+          g_variant_builder_add (&builder, "{s@v}", "subdir",
+                                 g_variant_new_variant (g_variant_new_string (opt_subpaths[0])));
+        else
+          g_variant_builder_add (&builder, "{s@v}", "subdirs",
+                                 g_variant_new_variant (g_variant_new_strv ((const char *const*) opt_subpaths, -1)));
+      }
     g_variant_builder_add (&builder, "{s@v}", "flags",
                            g_variant_new_variant (g_variant_new_int32 (pullflags)));
     if (refs_to_fetch)
@@ -240,6 +251,29 @@ ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **
       g_variant_builder_add (&builder, "{s@v}", "override-commit-ids",
                              g_variant_new_variant (g_variant_new_strv ((const char*const*)override_commit_ids->pdata, override_commit_ids->len)));
 
+    if (opt_http_headers)
+      {
+        GVariantBuilder hdr_builder;
+        g_variant_builder_init (&hdr_builder, G_VARIANT_TYPE ("a(ss)"));
+
+        for (char **iter = opt_http_headers; iter && *iter; iter++)
+          {
+            const char *kv = *iter;
+            const char *eq = strchr (kv, '=');
+            g_autofree char *key = NULL;
+            if (!eq)
+              {
+                g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                             "Missing '=' in --http-header");
+                goto out;
+              }
+            key = g_strndup (kv, eq - kv);
+            g_variant_builder_add (&hdr_builder, "(ss)", key, eq + 1);
+          }
+        g_variant_builder_add (&builder, "{s@v}", "http-headers",
+                               g_variant_new_variant (g_variant_builder_end (&hdr_builder)));
+      }
+
     if (!opt_dry_run)
       {
         if (console.is_tty)
@@ -257,7 +291,9 @@ ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **
                                               &console);
       }
 
-    if (!ostree_repo_pull_with_options (repo, remote, g_variant_builder_end (&builder),
+    options = g_variant_ref_sink (g_variant_builder_end (&builder));
+
+    if (!ostree_repo_pull_with_options (repo, remote, options,
                                         progress, cancellable, error))
       goto out;
 
@@ -272,7 +308,5 @@ ostree_builtin_pull (int argc, char **argv, GCancellable *cancellable, GError **
  out:
   if (signal_handler_id > 0)
     g_signal_handler_disconnect (repo, signal_handler_id);
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
