@@ -22,6 +22,7 @@
 
 #include "config.h"
 
+#include "ot-main.h"
 #include "ot-builtins.h"
 #include "ot-dump.h"
 #include "ostree.h"
@@ -32,13 +33,17 @@ static char* opt_print_variant_type;
 static char* opt_print_metadata_key;
 static char* opt_print_detached_metadata_key;
 static gboolean opt_raw;
+static char *opt_gpg_homedir;
+static char *opt_gpg_verify_remote;
 
 static GOptionEntry options[] = {
-  { "print-related", 0, 0, G_OPTION_ARG_NONE, &opt_print_related, "If given, show the \"related\" commits", NULL },
-  { "print-variant-type", 0, 0, G_OPTION_ARG_STRING, &opt_print_variant_type, "If given, argument should be a filename and it will be interpreted as this type", NULL },
-  { "print-metadata-key", 0, 0, G_OPTION_ARG_STRING, &opt_print_metadata_key, "Print string value of metadata key KEY for given commit", "KEY" },
-  { "print-detached-metadata-key", 0, 0, G_OPTION_ARG_STRING, &opt_print_detached_metadata_key, "Print string value of detached metadata key KEY for given commit", "KEY" },
+  { "print-related", 0, 0, G_OPTION_ARG_NONE, &opt_print_related, "Show the \"related\" commits", NULL },
+  { "print-variant-type", 0, 0, G_OPTION_ARG_STRING, &opt_print_variant_type, "Memory map OBJECT (in this case a filename) to the GVariant type string", "TYPE" },
+  { "print-metadata-key", 0, 0, G_OPTION_ARG_STRING, &opt_print_metadata_key, "Print string value of metadata key", "KEY" },
+  { "print-detached-metadata-key", 0, 0, G_OPTION_ARG_STRING, &opt_print_detached_metadata_key, "Print string value of detached metadata key", "KEY" },
   { "raw", 0, 0, G_OPTION_ARG_NONE, &opt_raw, "Show raw variant data" },
+  { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, "GPG Homedir to use when looking for keyrings", "HOMEDIR"},
+  { "gpg-verify-remote", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_verify_remote, "Use REMOTE name for GPG configuration", "REMOTE"},
   { NULL }
 };
 
@@ -48,12 +53,9 @@ do_print_variant_generic (const GVariantType *type,
                           GError **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFile *f = NULL;
-  gs_unref_variant GVariant *variant = NULL;
+  g_autoptr(GVariant) variant = NULL;
 
-  f = g_file_new_for_path (filename);
-
-  if (!ot_util_variant_map (f, type, TRUE, &variant, error))
+  if (!ot_util_variant_map_at (AT_FDCWD, filename, type, TRUE, &variant, error))
     goto out;
 
   ot_dump_variant (variant);
@@ -71,9 +73,9 @@ do_print_related (OstreeRepo  *repo,
 {
   gboolean ret = FALSE;
   const char *name;
-  gs_unref_variant GVariant *csum_v = NULL;
-  gs_unref_variant GVariant *variant = NULL;
-  gs_unref_variant GVariant *related = NULL;
+  g_autoptr(GVariant) csum_v = NULL;
+  g_autoptr(GVariant) variant = NULL;
+  g_autoptr(GVariant) related = NULL;
   GVariantIter *viter = NULL;
 
   if (!ostree_repo_load_variant (repo, OSTREE_OBJECT_TYPE_COMMIT,
@@ -87,7 +89,7 @@ do_print_related (OstreeRepo  *repo,
 
   while (g_variant_iter_loop (viter, "(&s@ay)", &name, &csum_v))
     {
-      gs_free char *checksum = ostree_checksum_from_bytes_v (csum_v);
+      g_autofree char *checksum = ostree_checksum_from_bytes_v (csum_v);
       g_print ("%s %s\n", name, checksum);
     }
   csum_v = NULL;
@@ -107,9 +109,9 @@ do_print_metadata_key (OstreeRepo     *repo,
                        GError        **error)
 {
   gboolean ret = FALSE;
-  gs_unref_variant GVariant *value = NULL;
-  gs_unref_variant GVariant *commit = NULL;
-  gs_unref_variant GVariant *metadata = NULL;
+  g_autoptr(GVariant) value = NULL;
+  g_autoptr(GVariant) commit = NULL;
+  g_autoptr(GVariant) metadata = NULL;
 
   if (!detached)
     {
@@ -153,7 +155,7 @@ print_object (OstreeRepo          *repo,
               const char          *checksum,
               GError             **error)
 {
-  gs_unref_variant GVariant *variant = NULL;
+  g_autoptr(GVariant) variant = NULL;
   OstreeDumpFlags flags = OSTREE_DUMP_NONE;
   gboolean ret = FALSE;
 
@@ -164,6 +166,55 @@ print_object (OstreeRepo          *repo,
     flags |= OSTREE_DUMP_RAW;
   ot_dump_object (objtype, checksum, variant, flags);
 
+  if (objtype == OSTREE_OBJECT_TYPE_COMMIT)
+    {
+      glnx_unref_object OstreeGpgVerifyResult *result = NULL;
+      GError *local_error = NULL;
+      g_autoptr(GFile) gpg_homedir = opt_gpg_homedir ? g_file_new_for_path (opt_gpg_homedir) : NULL;
+
+      if (opt_gpg_verify_remote)
+        {
+          result = ostree_repo_verify_commit_for_remote (repo, checksum, opt_gpg_verify_remote,
+                                                         NULL, &local_error);
+        }
+      else
+        {
+          result = ostree_repo_verify_commit_ext (repo, checksum,
+                                                  gpg_homedir, NULL, NULL,
+                                                  &local_error);
+        }
+
+      if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+        {
+          g_clear_error (&local_error);
+        }
+      else if (local_error != NULL)
+        {
+          g_propagate_error (error, local_error);
+          goto out;
+        }
+      else
+        {
+          GString *buffer;
+          guint n_sigs, ii;
+
+          n_sigs = ostree_gpg_verify_result_count_all (result);
+          g_print ("Found %u signature%s:\n", n_sigs, n_sigs == 1 ? "" : "s");
+
+          buffer = g_string_sized_new (256);
+
+          for (ii = 0; ii < n_sigs; ii++)
+            {
+              g_string_append_c (buffer, '\n');
+              ostree_gpg_verify_result_describe (result, ii, buffer, "  ",
+                                                 OSTREE_GPG_SIGNATURE_FORMAT_DEFAULT);
+            }
+
+          g_print ("%s", buffer->str);
+          g_string_free (buffer, TRUE);
+        }
+    }
+
   ret = TRUE;
 out:
   return ret;
@@ -199,17 +250,17 @@ print_if_found (OstreeRepo        *repo,
 }
 
 gboolean
-ostree_builtin_show (int argc, char **argv, OstreeRepo *repo, GCancellable *cancellable, GError **error)
+ostree_builtin_show (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
+  glnx_unref_object OstreeRepo *repo = NULL;
   gboolean ret = FALSE;
   const char *rev;
-  gs_free char *resolved_rev = NULL;
+  g_autofree char *resolved_rev = NULL;
 
   context = g_option_context_new ("OBJECT - Output a metadata object");
-  g_option_context_add_main_entries (context, options, NULL);
 
-  if (!g_option_context_parse (context, &argc, &argv, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NONE, &repo, cancellable, error))
     goto out;
 
   if (argc <= 1)
@@ -265,11 +316,11 @@ ostree_builtin_show (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
             goto out;
           if (!found)
             {
-              gs_unref_object GFileInfo *finfo = NULL;
-              gs_unref_variant GVariant *xattrs = NULL;
+              g_autoptr(GFileInfo) finfo = NULL;
+              g_autoptr(GVariant) xattrs = NULL;
               GFileType filetype;
               
-              if (!ostree_repo_load_file (repo, resolved_rev, NULL, &finfo, &xattrs,
+              if (!ostree_repo_load_file (repo, rev, NULL, &finfo, &xattrs,
                                           cancellable, error))
                 goto out;
 
@@ -297,7 +348,7 @@ ostree_builtin_show (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
               g_print ("Extended Attributes: ");
               if (xattrs)
                 {
-                  gs_free char *xattr_string = g_variant_print (xattrs, TRUE);
+                  g_autofree char *xattr_string = g_variant_print (xattrs, TRUE);
                   g_print ("{ %s }\n", xattr_string);
                 }
               else
@@ -310,7 +361,5 @@ ostree_builtin_show (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
  
   ret = TRUE;
  out:
-  if (context)
-    g_option_context_free (context);
   return ret;
 }
