@@ -83,7 +83,7 @@ int
 ostree_usage (OstreeCommand *commands,
               gboolean is_error)
 {
-  GOptionContext *context;
+  g_autoptr(GOptionContext) context = NULL;
   g_autofree char *help;
 
   context = ostree_option_context_new_with_commands (commands);
@@ -97,8 +97,6 @@ ostree_usage (OstreeCommand *commands,
   else
     g_print ("%s", help);
 
-  g_option_context_free (context);
-
   return (is_error ? 1 : 0);
 }
 
@@ -125,7 +123,6 @@ ostree_run (int    argc,
   GError *error = NULL;
   GCancellable *cancellable = NULL;
   const char *command_name = NULL;
-  g_autofree char *prgname = NULL;
   gboolean success = FALSE;
   int in, out;
 
@@ -173,7 +170,7 @@ ostree_run (int    argc,
 
   if (!command->fn)
     {
-      GOptionContext *context;
+      g_autoptr(GOptionContext) context = NULL;
       g_autofree char *help;
 
       context = ostree_option_context_new_with_commands (commands);
@@ -197,15 +194,9 @@ ostree_run (int    argc,
       help = g_option_context_get_help (context, FALSE, NULL);
       g_printerr ("%s", help);
 
-      g_option_context_free (context);
-
       goto out;
     }
 
-  prgname = g_strdup_printf ("%s %s", g_get_prgname (), command_name);
-  g_set_prgname (prgname);
-
-  
   if (!command->fn (argc, argv, cancellable, &error))
     goto out;
 
@@ -366,6 +357,13 @@ ostree_admin_option_context_parse (GOptionContext *context,
 
       g_print ("%s\n", deployment_path);
 
+      /* The g_autoptr, g_autofree etc. don't happen when we explicitly
+       * exit, making valgrind complain about leaks */
+      g_clear_object (&sysroot);
+      g_clear_object (&sysroot_path);
+      g_clear_object (&deployment_file);
+      g_clear_pointer (&deployments, g_ptr_array_unref);
+      g_clear_pointer (&deployment_path, g_free);
       exit (EXIT_SUCCESS);
     }
 
