@@ -35,6 +35,15 @@ assert_not_reached () {
 
 test_tmpdir=$(pwd)
 
+# Some tests look for specific English strings. Use a UTF-8 version
+# of the C (POSIX) locale if we have one, or fall back to POSIX
+# (https://sourceware.org/glibc/wiki/Proposals/C.UTF-8)
+if locale -a | grep C.UTF-8 >/dev/null; then
+  export LC_ALL=C.UTF-8
+else
+  export LC_ALL=C
+fi
+
 # Sanity check that we're in a tmpdir that has
 # just .testtmp (created by tap-driver for `make check`,
 # or nothing at all (as ginstest-runner does)
@@ -77,13 +86,16 @@ if test -n "${OT_TESTS_DEBUG:-}"; then
     set -x
 fi
 
+# This is substituted by the build for installed tests
+BUILT_WITH_ASAN=""
+
 if test -n "${OT_TESTS_VALGRIND:-}"; then
     CMD_PREFIX="env G_SLICE=always-malloc OSTREE_SUPPRESS_SYNCFS=1 valgrind -q --error-exitcode=1 --leak-check=full --num-callers=30 --suppressions=${test_srcdir}/glib.supp --suppressions=${test_srcdir}/ostree.supp"
 else
     # In some cases the LD_PRELOAD may cause obscure problems,
     # e.g. right now it breaks for me with -fsanitize=address, so
     # let's allow users to skip it.
-    if test -z "${OT_SKIP_READDIR_RAND:-}"; then
+    if test -z "${OT_SKIP_READDIR_RAND:-}" && test -z "${BUILT_WITH_ASAN:-}"; then
 	CMD_PREFIX="env LD_PRELOAD=${test_builddir}/libreaddir-rand.so"
     else
 	CMD_PREFIX=""
