@@ -17,14 +17,13 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-set -e
+set -euo pipefail
+
+echo "1..16"
 
 . $(dirname $0)/libtest.sh
 
-echo "1..1"
-
+# Exports OSTREE_SYSROOT so --sysroot not needed.
 setup_os_repository "archive-z2" "syslinux"
 
-echo "ok setup"
-
 . $(dirname $0)/admin-test.sh
