@@ -17,22 +17,21 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-set -e
+set -euo pipefail
 
 . $(dirname $0)/libtest.sh
 
 echo '1..11'
 
 setup_test_repository "archive-z2"
-echo "ok setup"
 
-. ${SRCDIR}/archive-test.sh
+. ${test_srcdir}/archive-test.sh
 
 cd ${test_tmpdir}
 mkdir repo2
 ${CMD_PREFIX} ostree --repo=repo2 init
 ${CMD_PREFIX} ostree --repo=repo2 remote add --set=gpg-verify=false aremote file://$(pwd)/repo test2
-ostree --repo=repo2 pull aremote
-ostree --repo=repo2 rev-parse aremote/test2
-ostree --repo=repo2 fsck
+${CMD_PREFIX} ostree --repo=repo2 pull aremote
+${CMD_PREFIX} ostree --repo=repo2 rev-parse aremote/test2
+${CMD_PREFIX} ostree --repo=repo2 fsck
 echo "ok pull with from file:/// uri"
