@@ -17,12 +17,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-set -e
+set -euo pipefail
 
 . $(dirname $0)/libtest.sh
 
 setup_fake_remote_repo1 "archive-z2"
 
-echo '1..2'
-
-. ${SRCDIR}/pull-test.sh
+. ${test_srcdir}/pull-test.sh
