@@ -20,22 +20,22 @@
 
 #include "config.h"
 
-#include "libgsystem.h"
+#include "libglnx.h"
 
 #include "ostree-varint.h"
 
 static void
 check_one_roundtrip (guint64    val)
 {
-  GString *buf = g_string_new (NULL);
+  g_autoptr(GString) buf = g_string_new (NULL);
   guint64 newval;
   gsize bytes_read;
 
   _ostree_write_varuint64 (buf, val);
   if (g_test_verbose ())
     {
-      gs_unref_variant GVariant *v = g_variant_new_from_data (G_VARIANT_TYPE ("ay"), buf->str, buf->len, TRUE, NULL, NULL);
-      gs_free char *data = g_variant_print (v, FALSE);
+      g_autoptr(GVariant) v = g_variant_new_from_data (G_VARIANT_TYPE ("ay"), buf->str, buf->len, TRUE, NULL, NULL);
+      g_autofree char *data = g_variant_print (v, FALSE);
       g_test_message ("%" G_GUINT64_FORMAT " -> %s", val, data);
     }
   g_assert (_ostree_read_varuint64 ((guint8*)buf->str, buf->len, &newval, &bytes_read));
